# -*- coding:utf-8 -*-
"""Parser for Twitter on Android."""

from __future__ import unicode_literals

from dfdatetime import java_time as dfdatetime_java_time

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.parsers import sqlite
from plaso.parsers.sqlite_plugins import interface


class TwitterAndroidContactEventData(events.EventData):
  """Twitter on Android contact event data.

  Attributes:
    identifier (int): contact row id.
    user_identifier (int): twitter account id.
    username (str): twitter account handler.
    name (str): twitter account name.
    description (str): twitter account profile description.
    web_url (str): twitter account profile url content.
    location (str): twitter account profile location content.
    followers (int): number of followers.
    friends (int): number of following.
    statuses (int): twitter account number of tweets.
    image_url (str): profile picture url.
  """

  DATA_TYPE = 'twitter:android:contact'

  def __init__(self):
    """Initializes event data."""
    super(TwitterAndroidContactEventData,
          self).__init__(data_type=self.DATA_TYPE)
    self.identifier = None
    self.user_identifier = None
    self.username = None
    self.name = None
    self.description = None
    self.web_url = None
    self.location = None
    self.followers = None
    self.friends = None
    self.statuses = None
    self.image_url = None


class TwitterAndroidStatusEventData(events.EventData):
  """Twitter on Android status event data.

  Attributes:
    identifier (int): status row identifier.
    author_identifier (int): twitter account identifier.
    username (str): twitter account handler.
    content (str): status content.
    favorited (int): favorited flag as 0/1 value.
    retweeted (int): retweeted flag as 0/1 value.
  """

  DATA_TYPE = 'twitter:android:status'

  def __init__(self):
    """Initializes event data."""
    super(TwitterAndroidStatusEventData,
          self).__init__(data_type=self.DATA_TYPE)
    self.identifier = None
    self.author_identifier = None
    self.username = None
    self.content = None
    self.favorited = None
    self.retweeted = None


class TwitterAndroidSearchEventData(events.EventData):
  """Twitter on Android search event data.

  Attributes:
    name (str): twitter name handler.
    search_query (str): search query.
  """

  DATA_TYPE = 'twitter:android:search'

  def __init__(self):
    """Initializes event data."""
    super(TwitterAndroidSearchEventData,
          self).__init__(data_type=self.DATA_TYPE)
    self.name = None
    self.search_query = None


class TwitterAndroidPlugin(interface.SQLitePlugin):
  """Parser plugin for Twitter on Android."""

  NAME = 'twitter_android'
  DESCRIPTION = 'Parser for Twitter on android database'

  QUERIES = [
      ('SELECT name, query, time FROM search_queries', 'ParseSearchRow'),
      (('SELECT statuses._id AS _id, statuses.author_id AS author_id, '
        'users.username AS username, statuses.content AS content, '
        'statuses.created AS time, statuses.favorited AS favorited, '
        'statuses.retweeted AS retweeted FROM statuses LEFT JOIN users ON '
        'statuses.author_id = users.user_id'), 'ParseStatusRow'),
      (('SELECT _id, user_id, username, name, profile_created, description, '
        'web_url, location, followers, friends, statuses, image_url, updated, '
        'friendship_time FROM users'), 'ParseContactRow')]

  REQUIRED_TABLES = frozenset(['search_queries', 'statuses', 'users'])

  SCHEMAS = [{
      'activities': (
          'CREATE TABLE activities (_id INTEGER PRIMARY KEY,type INT,event '
          'INT,created_at INT,hash INT,max_position INT,min_position '
          'INT,sources_size INT,source_type INT,sources BLOB,targets_size '
          'INT,target_type INT,targets BLOB,target_objects_size '
          'INT,target_object_type INT,target_objects BLOB,is_last INT,tag '
          'INT,magic_rec_id INT,UNIQUE (type, max_position) ON CONFLICT '
          'REPLACE)'),
      'ads_account_permissions': (
          'CREATE TABLE ads_account_permissions (_id INTEGER PRIMARY '
          'KEY,promotable_users BLOB,last_synced INT NOT NULL)'),
      'android_metadata': (
          'CREATE TABLE android_metadata (locale TEXT)'),
      'business_profiles': (
          'CREATE TABLE business_profiles (_id INTEGER PRIMARY KEY,user_id '
          'INT UNIQUE NOT NULL,business_profile BLOB,last_synced INT NOT '
          'NULL)'),
      'card_state': (
          'CREATE TABLE card_state (_id INTEGER PRIMARY KEY '
          'AUTOINCREMENT,card_status_id INT,card_id INT, card_state BLOB)'),
      'category_timestamp': (
          'CREATE TABLE category_timestamp (_id INTEGER PRIMARY '
          'KEY,cat_status_id INT NOT NULL,cat_tag INT NOT NULL,cat_timestamp '
          'INT NOT NULL)'),
      'clusters': (
          'CREATE TABLE clusters (_id INTEGER PRIMARY KEY,cl_cluster_id TEXT '
          'UNIQUE NOT NULL,cl_type INT,cl_title TEXT,cl_subtitle TEXT,cl_size '
          'INT,cl_timestamp INT,cl_content BLOB)'),
      'conversation_entries': (
          'CREATE TABLE conversation_entries (_id INTEGER PRIMARY '
          'KEY,entry_id INT UNIQUE NOT NULL,sort_entry_id INT UNIQUE NOT '
          'NULL,conversation_id TEXT,user_id INT,created INT,entry_type '
          'INT,data BLOB,request_id TEXT)'),
      'conversation_participants': (
          'CREATE TABLE conversation_participants (_id INTEGER PRIMARY '
          'KEY,conversation_id TEXT NOT NULL,user_id TEXT NOT NULL,join_time '
          'INT NOT NULL,participant_type INT NOT NULL)'),
      'conversations': (
          'CREATE TABLE conversations (_id INTEGER PRIMARY '
          'KEY,conversation_id TEXT UNIQUE NOT NULL,title TEXT,avatar_url '
          'TEXT,type INT,sort_event_id BIGINT,last_readable_event_id '
          'BIGINT,last_read_event_id BIGINT,sort_timestamp BIGINT,is_muted '
          'INT,min_event_id BIGINT,is_hidden INT,has_more INT,read_only INT)'),
      'cursors': (
          'CREATE TABLE cursors (_id INTEGER PRIMARY KEY,kind INT,type '
          'INT,owner_id INT,ref_id TEXT,next TEXT)'),
      'dismiss_info': (
          'CREATE TABLE dismiss_info(timeline_id INTEGER REFERENCES '
          'timeline(_id),feedback_action_id INTEGER REFERENCES '
          'feedback_action(_id),UNIQUE(timeline_id,feedback_action_id))'),
      'feedback_action': (
          'CREATE TABLE feedback_action(_id INTEGER PRIMARY KEY '
          'AUTOINCREMENT,feedback_type TEXT,prompt TEXT,confirmation '
          'TEXT,UNIQUE(feedback_type,prompt,confirmation))'),
      'list_mapping': (
          'CREATE TABLE list_mapping (_id INTEGER PRIMARY '
          'KEY,list_mapping_list_id TEXT,list_mapping_type '
          'INT,list_mapping_user_id INT,list_is_last INT)'),
      'locations': (
          'CREATE TABLE locations (_id INTEGER PRIMARY KEY,name TEXT,woeid '
          'INT,country TEXT,country_code TEXT)'),
      'moments': (
          'CREATE TABLE moments (_id INTEGER PRIMARY KEY,title TEXT NOT '
          'NULL,can_subscribe INT,is_live INT,is_sensitive '
          'INT,subcategory_string TEXT,subcategory_favicon_url '
          'TEXT,time_string TEXT,duration_string TEXT,is_subscribed '
          'INT,description TEXT NOT NULL,moment_url TEXT,num_subscribers '
          'INT,author_info BLOB,promoted_content BLOB)'),
      'moments_guide': (
          'CREATE TABLE moments_guide (_id INTEGER PRIMARY KEY,moment_id INT '
          'NOT NULL,section_id INT NOT NULL,tweet_id INT NOT NULL, crop_data '
          'BLOB,media_id INT,media_url TEXT,media_size BLOB,FOREIGN '
          'KEY(section_id) REFERENCES moments_sections(_id) ON DELETE '
          'CASCADE)'),
      'moments_guide_categories': (
          'CREATE TABLE moments_guide_categories (_id INTEGER PRIMARY '
          'KEY,category_id TEXT NOT NULL,is_default_category INT NOT '
          'NULL,category_name TEXT NOT NULL,fetch_timestamp INT NOT NULL)'),
      'moments_guide_user_states': (
          'CREATE TABLE moments_guide_user_states (_id INTEGER PRIMARY '
          'KEY,moment_id INT NOT NULL,is_read INT,is_updated INT,FOREIGN '
          'KEY(moment_id) REFERENCES moments(_id) ON DELETE CASCADE)'),
      'moments_pages': (
          'CREATE TABLE moments_pages (_id INTEGER PRIMARY KEY,moment_id INT '
          'NOT NULL,page_id TEXT,type BLOB,tweet_id INT,display_mode '
          'BLOB,page_number INT,crop_data BLOB,theme_data BLOB,media_id '
          'INT,media_size BLOB,media_url TEXT,last_read_timestamp INT,FOREIGN '
          'KEY(moment_id) REFERENCES moments(_id))'),
      'moments_sections': (
          'CREATE TABLE moments_sections (_id INTEGER PRIMARY '
          'KEY,section_title TEXT,section_type BLOB NOT NULL,section_group_id '
          'TEXT,section_group_type INT NOT NULL)'),
      'moments_visit_badge': (
          'CREATE TABLE moments_visit_badge (_id INTEGER PRIMARY '
          'KEY,moment_id INT UNIQUE NOT NULL,is_new_since_visit '
          'INT,is_updated_since_visit INT)'),
      'news': (
          'CREATE TABLE news (_id INTEGER PRIMARY KEY AUTOINCREMENT,country '
          'TEXT,language TEXT,topic_id INT,news_id TEXT,title TEXT,image_url '
          'TEXT,author_name TEXT,article_description TEXT,article_url '
          'TEXT,tweet_count INT,start_time INT,news_id_hash INT)'),
      'notifications': (
          'CREATE TABLE notifications (_id INTEGER PRIMARY KEY,type '
          'INT,notif_id INT,source_user_name TEXT,s_name TEXT,s_id '
          'INT,notif_txt TEXT,aggregation_data TEXT,notif_extra_data BLOB)'),
      'one_click': (
          'CREATE TABLE one_click (_id INTEGER PRIMARY KEY,topic '
          'TEXT,filter_name TEXT,filter_location TEXT,filter_follow INT)'),
      'order_history': (
          'CREATE TABLE order_history (_id INTEGER PRIMARY KEY,ordered_at INT '
          ',order_id INT ,data BLOB)'),
      'promoted_retry': (
          'CREATE TABLE promoted_retry(impression_id TEXT,event INT NOT '
          'NULL,is_earned INT NOT NULL,trend_id INT,num_retries INT NOT '
          'NULL,url TEXT,video_playlist_url TEXT,video_content_uuid '
          'TEXT,video_content_type TEXT,video_cta_url TEXT,video_cta_app_id '
          'TEXT,video_cta_app_name TEXT,card_event TEXT,PRIMARY '
          'KEY(impression_id,event,is_earned,trend_id))'),
      'prompts': (
          'CREATE TABLE prompts (_id INTEGER PRIMARY KEY,p_id INT,p_format '
          'TEXT,p_template TEXT,p_header TEXT,p_text TEXT,p_action_text '
          'TEXT,p_action_url TEXT,p_icon TEXT,p_background_image_url '
          'TEXT,p_persistence TEXT,p_entities BLOB,p_header_entities '
          'BLOB,p_status_id LONG,p_insertion_index INT,p_trigger TEXT)'),
      'rankings': (
          'CREATE TABLE rankings (_id INTEGER PRIMARY KEY '
          'AUTOINCREMENT,country TEXT,language TEXT,granularity TEXT,category '
          'TEXT,date INT)'),
      'search_queries': (
          'CREATE TABLE search_queries (_id INTEGER PRIMARY KEY,type INT,name '
          'TEXT NOT NULL,query TEXT NOT NULL,query_id INT,time INT,latitude '
          'REAL,longitude REAL,radius REAL,location TEXT,pc '
          'BLOB,cluster_titles BLOB)'),
      'search_results': (
          'CREATE TABLE search_results (_id INTEGER PRIMARY KEY,search_id '
          'INT,s_type INT,data_type INT,type_id INT,polled INT,data_id '
          'INT,related_data BLOB,cluster_id INT)'),
      'search_suggestion_metadata': (
          'CREATE TABLE search_suggestion_metadata (_id INTEGER PRIMARY '
          'KEY,type INT,last_update LONG)'),
      'status_groups': (
          'CREATE TABLE status_groups (_id INTEGER PRIMARY KEY,tweet_type INT '
          'DEFAULT 0,type INT,sender_id INT,owner_id INT,ref_id INT,tag '
          'INT,g_status_id INT,is_read INT,page INT,is_last INT,updated_at '
          'INT,timeline INT,pc BLOB,g_flags INT,preview_draft_id '
          'INT,preview_media BLOB,tweet_pivots BLOB)'),
      'status_metadata': (
          'CREATE TABLE status_metadata (_id INTEGER PRIMARY KEY,owner_id INT '
          'NOT NULL,status_id INT NOT NULL,status_group INT NOT '
          'NULL,status_group_tag INT NOT NULL,soc_type INT,soc_name '
          'TEXT,soc_second_name TEXT,soc_others_count INT,soc_fav_count '
          'INT,soc_rt_count INT,reason_icon_type TEXT,reason_text '
          'TEXT,scribe_component TEXT,scribe_data BLOB,highlights TEXT)'),
      'statuses': (
          'CREATE TABLE statuses (_id INTEGER PRIMARY KEY,status_id INT '
          'UNIQUE NOT NULL,author_id INT,content TEXT,source TEXT,created '
          'INT,in_r_user_id INT,in_r_status_id INT,favorited INT,latitude '
          'TEXT,longitude TEXT,place_data BLOB,entities TEXT,retweet_count '
          'INT,r_content TEXT,cards BLOB,flags INT,favorite_count INT,lang '
          'TEXT,supplemental_language TEXT,view_count INT,quoted_tweet_data '
          'BLOB,quoted_tweet_id INT,retweeted INT)'),
      'stories': (
          'CREATE TABLE stories ( _id INTEGER PRIMARY KEY,story_id '
          'TEXT,story_order INT,story_type INT,story_proof_type '
          'INT,story_proof_addl_count INT,data_type INT,data_id '
          'INT,story_is_read INT,story_meta_title TEXT,story_meta_subtitle '
          'TEXT,story_meta_query TEXT,story_meta_header_img_url '
          'TEXT,story_source TEXT,story_impression_info TEXT,story_tag INT)'),
      'timeline': (
          'CREATE TABLE timeline (_id INTEGER PRIMARY KEY '
          'AUTOINCREMENT,owner_id INT,type INT,sort_index INT,entity_id '
          'INT,entity_type INT,data_type INT,data_type_group '
          'INT,data_type_tag INT,timeline_tag TEXT,timeline_group_id '
          'INT,timeline_scribe_group_id INT,data_id INT,data BLOB,flags '
          'INT,updated_at INT,data_origin_id TEXT,is_last INT,is_read '
          'INT,scribe_content BLOB,timeline_moment_info BLOB,dismissed INT '
          'NOT NULL DEFAULT 0,dismiss_actions INT NOT NULL DEFAULT 0)'),
      'tokens': (
          'CREATE TABLE tokens (_id INTEGER PRIMARY KEY,text TEXT,weight '
          'INT,type INT,ref_id INT)'),
      'topics': (
          'CREATE TABLE topics (_id INTEGER PRIMARY KEY,ev_id TEXT UNIQUE NOT '
          'NULL,ev_type INT,ev_query TEXT NOT NULL,ev_seed_hashtag '
          'TEXT,ev_title STRING,ev_subtitle STRING,ev_view_url '
          'STRING,ev_status STRING,ev_image_url TEXT,ev_explanation '
          'TEXT,ev_tweet_count INT,ev_start_time INT,ev_owner_id INT,ev_pc '
          'BLOB,ev_content BLOB,ev_hash INT)'),
      'user_groups': (
          'CREATE TABLE user_groups (_id INTEGER PRIMARY KEY,type INT,tag '
          'INT,rank INT,owner_id INT,user_id INT,is_last INT,pc BLOB,g_flags '
          'INT)'),
      'user_metadata': (
          'CREATE TABLE user_metadata (_id INTEGER PRIMARY KEY,owner_id INT '
          'NOT NULL,user_id INT NOT NULL,user_group_type INT NOT '
          'NULL,user_group_tag INT NOT NULL,soc_type INT,soc_name '
          'TEXT,soc_follow_count INT,user_title TEXT,token TEXT)'),
      'users': (
          'CREATE TABLE users (_id INTEGER PRIMARY KEY,user_id INT UNIQUE NOT '
          'NULL,username TEXT,name TEXT,description TEXT,web_url '
          'TEXT,bg_color INT,location TEXT,structured_location '
          'BLOB,user_flags INT,followers INT,fast_followers INT DEFAULT '
          '0,friends INT,statuses INT,profile_created INT,image_url TEXT,hash '
          'INT,updated INT,friendship INT,friendship_time INT,favorites INT '
          'DEFAULT 0,header_url TEXT,description_entities BLOB,url_entities '
          'BLOB,media_count INT,extended_profile_fields BLOB,pinned_tweet_id '
          'INT,link_color INT,advertiser_type TEXT,business_profile_state '
          'TEXT)')}]

  def ParseSearchRow(self, parser_mediator, query, row, **unused_kwargs):
    """Parses a search row from the database.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      query (str): query that created the row.
      row (sqlite3.Row): row resulting from query.
    """
    query_hash = hash(query)

    event_data = TwitterAndroidSearchEventData()
    event_data.query = query
    event_data.name = self._GetRowValue(query_hash, row, 'name')
    event_data.search_query = self._GetRowValue(query_hash, row, 'query')

    timestamp = self._GetRowValue(query_hash, row, 'time')
    if timestamp:
      date_time = dfdatetime_java_time.JavaTime(timestamp=timestamp)
      event = time_events.DateTimeValuesEvent(
          date_time, definitions.TIME_DESCRIPTION_CREATION)
      parser_mediator.ProduceEventWithEventData(event, event_data)

  def ParseStatusRow(self, parser_mediator, query, row, **unused_kwargs):
    """Parses a status row from the database.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      query (str): query that created the row.
      row (sqlite3.Row): row resulting from query.
    """
    query_hash = hash(query)

    event_data = TwitterAndroidStatusEventData()
    event_data.query = query
    event_data.identifier = self._GetRowValue(query_hash, row, '_id')
    event_data.author_identifier = self._GetRowValue(
        query_hash, row, 'author_id')
    event_data.username = self._GetRowValue(query_hash, row, 'username')
    event_data.content = self._GetRowValue(query_hash, row, 'content')
    event_data.favorited = self._GetRowValue(query_hash, row, 'favorited')
    event_data.retweeted = self._GetRowValue(query_hash, row, 'retweeted')

    timestamp = self._GetRowValue(query_hash, row, 'time')
    if timestamp:
      date_time = dfdatetime_java_time.JavaTime(timestamp=timestamp)
      event = time_events.DateTimeValuesEvent(
          date_time, definitions.TIME_DESCRIPTION_CREATION)
      parser_mediator.ProduceEventWithEventData(event, event_data)

  def ParseContactRow(self, parser_mediator, query, row, **unused_kwargs):
    """Parses a status row from the database.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      query (str): query that created the row.
      row (sqlite3.Row): row resulting from query.
    """
    query_hash = hash(query)

    event_data = TwitterAndroidContactEventData()
    event_data.query = query
    event_data.identifier = self._GetRowValue(query_hash, row, '_id')
    event_data.user_identifier = self._GetRowValue(query_hash, row, 'user_id')
    event_data.username = self._GetRowValue(query_hash, row, 'username')
    event_data.name = self._GetRowValue(query_hash, row, 'name')
    event_data.description = self._GetRowValue(query_hash, row, 'description')
    event_data.web_url = self._GetRowValue(query_hash, row, 'web_url')
    event_data.location = self._GetRowValue(query_hash, row, 'location')
    event_data.followers = self._GetRowValue(query_hash, row, 'followers')
    event_data.friends = self._GetRowValue(query_hash, row, 'friends')
    event_data.statuses = self._GetRowValue(query_hash, row, 'statuses')
    event_data.image_url = self._GetRowValue(query_hash, row, 'image_url')

    timestamp = self._GetRowValue(query_hash, row, 'profile_created')
    if timestamp:
      date_time = dfdatetime_java_time.JavaTime(timestamp=timestamp)
      event = time_events.DateTimeValuesEvent(
          date_time, definitions.TIME_DESCRIPTION_CREATION)
      parser_mediator.ProduceEventWithEventData(event, event_data)

    timestamp = self._GetRowValue(query_hash, row, 'updated')
    if timestamp:
      date_time = dfdatetime_java_time.JavaTime(timestamp=timestamp)
      event = time_events.DateTimeValuesEvent(
          date_time, definitions.TIME_DESCRIPTION_UPDATE)
      parser_mediator.ProduceEventWithEventData(event, event_data)

    timestamp = self._GetRowValue(query_hash, row, 'friendship_time')
    if timestamp:
      date_time = dfdatetime_java_time.JavaTime(timestamp=timestamp)
      event = time_events.DateTimeValuesEvent(
          date_time, definitions.TIME_DESCRIPTION_MODIFICATION)
      parser_mediator.ProduceEventWithEventData(event, event_data)


sqlite.SQLiteParser.RegisterPlugin(TwitterAndroidPlugin)



# -*- coding: utf-8 -*-
"""The MSIE zone settings event formatter."""

from __future__ import unicode_literals

from plaso.formatters import interface
from plaso.formatters import manager


class MSIEZoneSettingsEventFormatter(interface.EventFormatter):
  """Formatter for a MSIE zone settings event."""

  DATA_TYPE = 'windows:registry:msie_zone_settings'

  FORMAT_STRING = '[{key_path}] {settings}'
  FORMAT_STRING_ALTERNATIVE = '{settings}'

  SOURCE_LONG = 'Registry Key'
  SOURCE_SHORT = 'REG'


manager.FormattersManager.RegisterFormatter(MSIEZoneSettingsEventFormatter)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Parser test for MacOS Cups IPP Log files."""

from __future__ import unicode_literals

import io
import unittest

from dfvfs.helpers import fake_file_system_builder
from dfvfs.path import fake_path_spec

from plaso.formatters import cups_ipp as _  # pylint: disable=unused-import
from plaso.lib import definitions
from plaso.lib import errors
from plaso.parsers import cups_ipp

from tests.parsers import test_lib


class CupsIppParserTest(test_lib.ParserTestCase):
  """Tests for MacOS Cups IPP parser."""

  # pylint: disable=protected-access

  _ATTRIBUTES_GROUP_DATA = bytes(bytearray([
      0x01, 0x47, 0x00, 0x12, 0x61, 0x74, 0x74, 0x72, 0x69, 0x62, 0x75, 0x74,
      0x65, 0x73, 0x2d, 0x63, 0x68, 0x61, 0x72, 0x73, 0x65, 0x74, 0x00, 0x05,
      0x75, 0x74, 0x66, 0x2d, 0x38, 0x03]))

  def _CreateAttributeTestData(self, parser, tag_value, name, value_data):
    """Creates attribute test data.

    Args:
      parser (CupsIppParser): CUPS IPP parser.
      tag_value (int): value of the attribute tag.
      name (str): name of the attribute.
      value_data (bytes): data of the attribute value.

    Returns:
      bytes: attribute test data.
    """
    attribute_map = parser._GetDataTypeMap('cups_ipp_attribute')

    attribute = attribute_map.CreateStructureValues(
        tag_value=tag_value, name_size=len(name), name=name,
        value_data_size=len(value_data), value_data=value_data)
    return attribute_map.FoldByteStream(attribute)

  def _CreateDateTimeValueData(self, parser):
    """Created date time value test data.

    Args:
      parser (CupsIppParser): CUPS IPP parser.

    Returns:
      bytes: date time value test data.
    """
    datetime_map = parser._GetDataTypeMap('cups_ipp_datetime_value')

    datetime = datetime_map.CreateStructureValues(
        year=2018, month=11, day_of_month=27, hours=16, minutes=41, seconds=51,
        deciseconds=5, direction_from_utc=ord('+'), hours_from_utc=1,
        minutes_from_utc=0)
    return datetime_map.FoldByteStream(datetime)

  def testGetStringValue(self):
    """Tests the _GetStringValue function."""
    parser = cups_ipp.CupsIppParser()

    string_value = parser._GetStringValue({}, 'test')
    self.assertIsNone(string_value)

    string_value = parser._GetStringValue({'test': ['1', '2,3', '4']}, 'test')
    self.assertEqual(string_value, '1, "2,3", 4')

  def testParseAttribute(self):
    """Tests the _ParseAttribute function."""
    parser = cups_ipp.CupsIppParser()

    attribute_data = self._CreateAttributeTestData(
        parser, 0x00, 'test', b'\x12')
    file_object = io.BytesIO(attribute_data)

    name, value = parser._ParseAttribute(file_object)
    self.assertEqual(name, 'test')
    self.assertEqual(value, b'\x12')

    # Test with attribute data too small.
    file_object = io.BytesIO(attribute_data[:-1])

    with self.assertRaises(errors.ParseError):
      parser._ParseAttribute(file_object)

    # Test attribute with integer value.
    attribute_data = self._CreateAttributeTestData(
        parser, 0x21, 'int', b'\x12\x34\x56\x78')
    file_object = io.BytesIO(attribute_data)

    name, value = parser._ParseAttribute(file_object)
    self.assertEqual(name, 'int')
    self.assertEqual(value, 0x12345678)

    # Test attribute with boolean value.
    attribute_data = self._CreateAttributeTestData(
        parser, 0x22, 'bool', b'\x01')
    file_object = io.BytesIO(attribute_data)

    name, value = parser._ParseAttribute(file_object)
    self.assertEqual(name, 'bool')
    self.assertEqual(value, True)

    # Test attribute with date time value.
    datetime_data = self._CreateDateTimeValueData(parser)
    attribute_data = self._CreateAttributeTestData(
        parser, 0x31, 'datetime', datetime_data)
    file_object = io.BytesIO(attribute_data)

    name, value = parser._ParseAttribute(file_object)
    self.assertEqual(name, 'datetime')
    self.assertIsNotNone(value)
    self.assertEqual(value.year, 2018)

    # Test attribute with string without language.
    attribute_data = self._CreateAttributeTestData(
        parser, 0x42, 'string', b'NOLANG')
    file_object = io.BytesIO(attribute_data)

    name, value = parser._ParseAttribute(file_object)
    self.assertEqual(name, 'string')
    self.assertEqual(value, 'NOLANG')

    # Test attribute with ASCII string and tag value charset.
    attribute_data = self._CreateAttributeTestData(
        parser, 0x47, 'charset', b'utf8')
    file_object = io.BytesIO(attribute_data)

    name, value = parser._ParseAttribute(file_object)
    self.assertEqual(name, 'charset')
    self.assertEqual(value, 'utf8')

  def testParseAttributesGroup(self):
    """Tests the _ParseAttributesGroup function."""
    parser = cups_ipp.CupsIppParser()

    file_object = io.BytesIO(self._ATTRIBUTES_GROUP_DATA)

    name_value_pairs = list(parser._ParseAttributesGroup(file_object))
    self.assertEqual(name_value_pairs, [('attributes-charset', 'utf-8')])

    # Test with unsupported attributes groups start tag value.
    file_object = io.BytesIO(b''.join([
        b'\xff', self._ATTRIBUTES_GROUP_DATA[1:]]))

    with self.assertRaises(errors.ParseError):
      list(parser._ParseAttributesGroup(file_object))

  def testParseBooleanValue(self):
    """Tests the _ParseBooleanValue function."""
    parser = cups_ipp.CupsIppParser()

    boolean_value = parser._ParseBooleanValue(b'\x00')
    self.assertFalse(boolean_value)

    boolean_value = parser._ParseBooleanValue(b'\x01')
    self.assertTrue(boolean_value)

    # Test with unsupported data.
    with self.assertRaises(errors.ParseError):
      parser._ParseBooleanValue(b'\x02')

  def testParseDateTimeValue(self):
    """Tests the _ParseDateTimeValue function."""
    parser = cups_ipp.CupsIppParser()

    datetime_data = self._CreateDateTimeValueData(parser)

    datetime_value = parser._ParseDateTimeValue(datetime_data, 0)
    self.assertIsNotNone(datetime_value)
    self.assertEqual(datetime_value.year, 2018)

    # Test with data too small.
    with self.assertRaises(errors.ParseError):
      parser._ParseDateTimeValue(datetime_data[:-1], 0)

  def testParseIntegerValue(self):
    """Tests the _ParseIntegerValue function."""
    parser = cups_ipp.CupsIppParser()

    integer_value = parser._ParseIntegerValue(b'\x00\x00\x00\x01', 0)
    self.assertEqual(integer_value, 1)

    # Test with data too small.
    with self.assertRaises(errors.ParseError):
      parser._ParseIntegerValue(b'\x01\x00\x00', 0)

  def testParseHeader(self):
    """Tests the _ParseHeader function."""
    file_system_builder = fake_file_system_builder.FakeFileSystemBuilder()
    file_system_builder.AddFile('/cups_ipp', b'')

    test_path_spec = fake_path_spec.FakePathSpec(location='/cups_ipp')
    test_file_entry = file_system_builder.file_system.GetFileEntryByPathSpec(
        test_path_spec)

    storage_writer = self._CreateStorageWriter()
    parser_mediator = self._CreateParserMediator(
        storage_writer, file_entry=test_file_entry)

    parser = cups_ipp.CupsIppParser()
    header_map = parser._GetDataTypeMap('cups_ipp_header')

    header = header_map.CreateStructureValues(
        major_version=1, minor_version=1, operation_identifier=5,
        request_identifier=0)
    header_data = header_map.FoldByteStream(header)
    file_object = io.BytesIO(header_data)

    parser._ParseHeader(parser_mediator, file_object)

    # Test with header data too small.
    file_object = io.BytesIO(header_data[:-1])

    with self.assertRaises(errors.UnableToParseFile):
      parser._ParseHeader(parser_mediator, file_object)

    # Test with unsupported format version.
    header = header_map.CreateStructureValues(
        major_version=99, minor_version=1, operation_identifier=5,
        request_identifier=0)
    header_data = header_map.FoldByteStream(header)
    file_object = io.BytesIO(header_data)

    with self.assertRaises(errors.UnableToParseFile):
      parser._ParseHeader(parser_mediator, file_object)

    # Test with unsupported operation identifier.
    header = header_map.CreateStructureValues(
        major_version=1, minor_version=1, operation_identifier=99,
        request_identifier=0)
    header_data = header_map.FoldByteStream(header)
    file_object = io.BytesIO(header_data)

    parser._ParseHeader(parser_mediator, file_object)

  def testParseFileObject(self):
    """Tests the ParseFileObject function."""
    parser = cups_ipp.CupsIppParser()

    header_map = parser._GetDataTypeMap('cups_ipp_header')

    header = header_map.CreateStructureValues(
        major_version=1, minor_version=1, operation_identifier=5,
        request_identifier=0)
    header_data = header_map.FoldByteStream(header)

    storage_writer = self._CreateStorageWriter()
    parser_mediator = self._CreateParserMediator(storage_writer)

    file_object = io.BytesIO(b''.join([
        header_data, self._ATTRIBUTES_GROUP_DATA]))

    parser.ParseFileObject(parser_mediator, file_object)

    self.assertEqual(storage_writer.number_of_warnings, 0)
    self.assertEqual(storage_writer.number_of_events, 0)

    # Test with attribute group data too small.
    storage_writer = self._CreateStorageWriter()
    parser_mediator = self._CreateParserMediator(storage_writer)

    file_object = io.BytesIO(b''.join([
        header_data, self._ATTRIBUTES_GROUP_DATA[:-1]]))

    parser.ParseFileObject(parser_mediator, file_object)

    self.assertEqual(storage_writer.number_of_warnings, 1)
    self.assertEqual(storage_writer.number_of_events, 0)

    # Test attribute with date time value.
    datetime_data = self._CreateDateTimeValueData(parser)
    attribute_data = self._CreateAttributeTestData(
        parser, 0x31, 'date-time-at-creation', datetime_data)

    storage_writer = self._CreateStorageWriter()
    parser_mediator = self._CreateParserMediator(storage_writer)

    file_object = io.BytesIO(b''.join([
        header_data, b'\x01', attribute_data, b'\x03']))

    parser.ParseFileObject(parser_mediator, file_object)

    self.assertEqual(storage_writer.number_of_warnings, 0)
    self.assertEqual(storage_writer.number_of_events, 1)

  def testParse(self):
    """Tests the Parse function."""
    # TODO: only tested against MacOS Cups IPP (Version 2.0)
    parser = cups_ipp.CupsIppParser()
    storage_writer = self._ParseFile(['mac_cups_ipp'], parser)

    self.assertEqual(storage_writer.number_of_warnings, 0)
    self.assertEqual(storage_writer.number_of_events, 3)

    events = list(storage_writer.GetSortedEvents())

    event = events[0]

    self.CheckTimestamp(event.timestamp, '2013-11-03 18:07:21.000000')
    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_CREATION)

    event_data = self._GetEventDataOfEvent(storage_writer, event)
    self.assertEqual(event_data.application, 'LibreOffice')
    self.assertEqual(event_data.job_name, 'Assignament 1')
    self.assertEqual(event_data.computer_name, 'localhost')
    self.assertEqual(event_data.copies, 1)
    self.assertEqual(event_data.doc_type, 'application/pdf')
    self.assertEqual(
        event_data.job_id, 'urn:uuid:d51116d9-143c-3863-62aa-6ef0202de49a')
    self.assertEqual(event_data.owner, 'Joaquin Moreno Garijo')
    self.assertEqual(event_data.user, 'moxilo')
    self.assertEqual(event_data.printer_id, 'RHULBW')
    self.assertEqual(event_data.uri, 'ipp://localhost:631/printers/RHULBW')

    expected_message = (
        'User: moxilo '
        'Owner: Joaquin Moreno Garijo '
        'Job Name: Assignament 1 '
        'Application: LibreOffice '
        'Printer: RHULBW')
    expected_short_message = 'Job Name: Assignament 1'
    self._TestGetMessageStrings(
        event_data, expected_message, expected_short_message)

    event = events[1]

    self.CheckTimestamp(event.timestamp, '2013-11-03 18:07:21.000000')
    self.assertEqual(event.timestamp_desc, definitions.TIME_DESCRIPTION_START)

    event = events[2]

    self.CheckTimestamp(event.timestamp, '2013-11-03 18:07:32.000000')
    self.assertEqual(event.timestamp_desc, definitions.TIME_DESCRIPTION_END)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""File system stat object parser."""

from __future__ import unicode_literals

from dfdatetime import posix_time as dfdatetime_posix_time
from dfvfs.lib import definitions as dfvfs_definitions

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.parsers import interface
from plaso.parsers import manager


class FileStatEventData(events.EventData):
  """File system stat event data.

  Attributes:
    file_entry_type (int): dfVFS file entry type.
    file_size (int): file size in bytes.
    file_system_type (str): file system type.
    inode (int): inode of the file related to the event.
    is_allocated (bool): True if the file is allocated.
  """

  DATA_TYPE = 'fs:stat'

  def __init__(self):
    """Initializes event data."""
    super(FileStatEventData, self).__init__(data_type=self.DATA_TYPE)
    self.file_entry_type = None
    self.file_size = None
    self.file_system_type = None
    self.inode = None
    self.is_allocated = None


class FileStatParser(interface.FileEntryParser):
  """Parses file system stat object."""

  NAME = 'filestat'
  DESCRIPTION = 'Parser for file system stat information.'

  _TIMESTAMP_DESCRIPTIONS = {
      'bkup_time': definitions.TIME_DESCRIPTION_BACKUP,
      'dtime': definitions.TIME_DESCRIPTION_DELETED,
  }

  def _GetFileSystemTypeFromFileEntry(self, file_entry):
    """Retrieves the file system type indicator of a file entry.

    Args:
      file_entry (dfvfs.FileEntry): a file entry.

    Returns:
      str: file system type.
    """
    if file_entry.type_indicator != dfvfs_definitions.TYPE_INDICATOR_TSK:
      return file_entry.type_indicator

    # TODO: Implement fs_type in dfVFS and remove this implementation
    # once that is in place.
    file_system = file_entry.GetFileSystem()
    fs_info = file_system.GetFsInfo()
    if fs_info.info:
      type_string = '{0!s}'.format(fs_info.info.ftype)
      if type_string.startswith('TSK_FS_TYPE_'):
        type_string = type_string[12:]
      if type_string.endswith('_DETECT'):
        type_string = type_string[:-7]

    return type_string

  def ParseFileEntry(self, parser_mediator, file_entry):
    """Parses a file entry.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      file_entry (dfvfs.FileEntry): a file entry.
    """
    stat_object = file_entry.GetStat()
    if not stat_object:
      return

    file_system_type = self._GetFileSystemTypeFromFileEntry(file_entry)

    event_data = FileStatEventData()
    event_data.file_entry_type = stat_object.type
    event_data.file_size = getattr(stat_object, 'size', None)
    event_data.file_system_type = file_system_type
    event_data.is_allocated = file_entry.IsAllocated()

    if file_entry.access_time:
      event = time_events.DateTimeValuesEvent(
          file_entry.access_time, definitions.TIME_DESCRIPTION_LAST_ACCESS)
      parser_mediator.ProduceEventWithEventData(event, event_data)

    if file_entry.creation_time:
      event = time_events.DateTimeValuesEvent(
          file_entry.creation_time, definitions.TIME_DESCRIPTION_CREATION)
      parser_mediator.ProduceEventWithEventData(event, event_data)

    if file_entry.change_time:
      event = time_events.DateTimeValuesEvent(
          file_entry.change_time, definitions.TIME_DESCRIPTION_CHANGE)
      parser_mediator.ProduceEventWithEventData(event, event_data)

    if file_entry.modification_time:
      event = time_events.DateTimeValuesEvent(
          file_entry.modification_time,
          definitions.TIME_DESCRIPTION_MODIFICATION)
      parser_mediator.ProduceEventWithEventData(event, event_data)

    for time_attribute, usage in self._TIMESTAMP_DESCRIPTIONS.items():
      posix_time = getattr(stat_object, time_attribute, None)
      if posix_time is None:
        continue

      nano_time_attribute = '{0:s}_nano'.format(time_attribute)
      nano_time_attribute = getattr(stat_object, nano_time_attribute, None)

      timestamp = posix_time * 1000000
      if nano_time_attribute is not None:
        # Note that the _nano values are in intervals of 100th nano seconds.
        micro_time_attribute, _ = divmod(nano_time_attribute, 10)
        timestamp += micro_time_attribute

      # TSK will return 0 if the timestamp is not set.
      if (file_entry.type_indicator == dfvfs_definitions.TYPE_INDICATOR_TSK and
          not timestamp):
        continue

      date_time = dfdatetime_posix_time.PosixTimeInMicroseconds(
          timestamp=timestamp)
      event = time_events.DateTimeValuesEvent(date_time, usage)
      parser_mediator.ProduceEventWithEventData(event, event_data)


manager.ParsersManager.RegisterParser(FileStatParser)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for Twitter on Android plugin."""

from __future__ import unicode_literals

import unittest

from plaso.lib import definitions
from plaso.parsers.sqlite_plugins import twitter_android

from tests.parsers.sqlite_plugins import test_lib


class TwitterAndroidTest(test_lib.SQLitePluginTestCase):
  """Tests for Twitter on Android database plugin."""

  def testProcess(self):
    """Test the Process function on a Twitter Android file."""
    plugin = twitter_android.TwitterAndroidPlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['twitter_android.db'], plugin)

    # We should have 850 events in total.
    self.assertEqual(850, storage_writer.number_of_events)

    events = list(storage_writer.GetSortedEvents())

    # Test status event data
    event = events[482]

    self.CheckTimestamp(event.timestamp, '2015-12-02 17:47:17.000000')
    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_CREATION)

    event_data = self._GetEventDataOfEvent(storage_writer, event)

    expected_content = (
        '@CarolMovie wins BEST PICTURE at #NYFCC!!! CONGRATS #TeamCarol!!! '
        'Love love! #carolfilm https://t.co/ycy9cHPLZ7')

    self.assertEqual(event_data.author_identifier, 2730978846)
    self.assertEqual(event_data.content, expected_content)
    self.assertEqual(event_data.favorited, 0)
    self.assertEqual(event_data.identifier, 4)
    self.assertEqual(event_data.retweeted, 0)
    self.assertEqual(event_data.username, 'CarolMovieFans')

    expected_message = (
        'User: CarolMovieFans Status: @CarolMovie wins BEST PICTURE at '
        '#NYFCC!!! CONGRATS #TeamCarol!!! Love love! #carolfilm '
        'https://t.co/ycy9cHPLZ7 Favorited: No Retweeted: No')

    expected_short_message = (
        'User: CarolMovieFans Status: @CarolMovie wins BEST PICTURE at '
        '#NYFCC!!! CONGR...')

    self._TestGetMessageStrings(
        event_data, expected_message, expected_short_message)

    # Test search event data
    event = events[837]
    self.CheckTimestamp(event.timestamp, '2015-12-02 20:49:38.153000')
    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_CREATION)

    event_data = self._GetEventDataOfEvent(storage_writer, event)
    self.assertEqual(event_data.name, 'rosegold')
    self.assertEqual(event_data.search_query, 'rosegold')

    expected_message = 'Name: rosegold Query: rosegold'

    expected_short_message = 'Query: rosegold'

    self._TestGetMessageStrings(
        event_data, expected_message, expected_short_message)

    # Test all three timestamps for contact event data.

    # Test profile creation time and event.
    event = events[24]
    self.CheckTimestamp(event.timestamp, '2008-06-03 18:30:55.000000')
    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_CREATION)

    event_data = self._GetEventDataOfEvent(storage_writer, event)
    self.assertEqual(event_data.identifier, 62)
    self.assertEqual(event_data.user_identifier, 14995801)
    self.assertEqual(event_data.username, 'timbuk2')
    self.assertEqual(event_data.name, 'Timbuk2')

    expected_description = (
        'Started in a San Francisco by bike messenger Rob Honeycutt, Timbuk2 '
        'has been making tough as hell messenger bags, backpacks and travel '
        'bags since 1989.')

    self.assertEqual(event_data.description, expected_description)
    self.assertEqual(event_data.web_url, 'http://t.co/Z0MZo7f2ne')
    self.assertEqual(event_data.location, 'San Francisco, CA')
    self.assertEqual(event_data.followers, 23582)
    self.assertEqual(event_data.friends, 2725)
    self.assertEqual(event_data.statuses, 18937)

    expected_image_url = (
        'https://pbs.twimg.com/profile_images/461846147129024512/'
        'FOKZJ7hB_normal.jpeg')

    self.assertEqual(event_data.image_url, expected_image_url)

    expected_message = (
        'Screen name: timbuk2 Profile picture URL: https://pbs.twimg.com/'
        'profile_images/461846147129024512/FOKZJ7hB_normal.jpeg Name: Timbuk2 '
        'Location: San Francisco, CA Description: Started in a San Francisco '
        'by bike messenger Rob Honeycutt, Timbuk2 has been making tough as '
        'hell messenger bags, backpacks and travel bags since 1989. URL: '
        'http://t.co/Z0MZo7f2ne Number of followers: 23582 Number of tweets: '
        '18937')

    expected_short_message = (
        'Screen name: timbuk2 Description: Started in a San Francisco by bike '
        'messenge...')

    self._TestGetMessageStrings(
        event_data, expected_message, expected_short_message)

    # Test friended time.
    event = events[581]
    self.CheckTimestamp(event.timestamp, '2015-12-02 20:48:32.382000')
    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_MODIFICATION)

    # Test profile update time.
    event = events[806]
    self.CheckTimestamp(event.timestamp, '2015-12-02 20:49:33.349000')
    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_UPDATE)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Advanced Packaging Tool (APT) History log parser."""

from __future__ import unicode_literals

import unittest

from plaso.formatters import apt_history as _  # pylint: disable=unused-import
from plaso.lib import errors
from plaso.parsers import apt_history

from tests.parsers import test_lib


class APTHistoryLogUnitTest(test_lib.ParserTestCase):
  """Tests for the APT History log parser.

  Since APT History logs record in local time, these tests assume that the local
  timezone is set to UTC.
  """

  def testParseLog(self):
    """Tests the Parse function on apt_history.log."""
    parser = apt_history.APTHistoryLogParser()
    storage_writer = self._ParseFile(['apt_history.log'], parser)

    self.assertEqual(storage_writer.number_of_warnings, 0)
    self.assertEqual(storage_writer.number_of_events, 10)

    events = list(storage_writer.GetEvents())

    event = events[0]

    self.CheckTimestamp(event.timestamp, '2019-07-10 16:38:08.000000')

    event = events[1]

    self.CheckTimestamp(event.timestamp, '2019-07-10 16:38:12.000000')

    event = events[2]
    event_data = self._GetEventDataOfEvent(storage_writer, event)

    self.CheckTimestamp(event.timestamp, '2019-07-11 12:20:55.000000')

    expected_message = (
        'Install: libmpc3:amd64 (1.0.3-1+b2, automatic), '
        'manpages:amd64 (4.10-2, automatic), '
        'libmpx2:amd64 (6.3.0-18+deb9u1, automatic), '
        'python3-dev:amd64 (3.5.3-1), '
        'python2.7-dev:amd64 (2.7.13-2+deb9u3, automatic), '
        'libdbus-1-3:amd64 (1.10.28-0+deb9u1, automatic), '
        'linux-libc-dev:amd64 (4.9.168-1+deb9u3, automatic), '
        'python-xdg:amd64 (0.25-4, automatic), '
        'libmpfr4:amd64 (3.1.5-1, automatic), '
        'sgml-base:amd64 (1.29, automatic), '
        'libgirepository-1.0-1:amd64 (1.50.0-1+b1, automatic), '
        'libfakeroot:amd64 (1.21-3.1, automatic), '
        'libc6-dev:amd64 (2.24-11+deb9u4, automatic), '
        'rename:amd64 (0.20-4, automatic), '
        'git-man:amd64 (1:2.11.0-3+deb9u4, automatic), '
        'xdg-user-dirs:amd64 (0.15-2+b1, automatic), '
        'libexpat1-dev:amd64 (2.2.0-2+deb9u2, automatic), '
        'cpp-6:amd64 (6.3.0-18+deb9u1, automatic), '
        'dbus:amd64 (1.10.28-0+deb9u1, automatic), '
        'libalgorithm-diff-perl:amd64 (1.19.03-1, automatic), '
        'python-gi:amd64 (3.22.0-2, automatic), '
        'libalgorithm-merge-perl:amd64 (0.08-3, automatic), '
        'libicu57:amd64 (57.1-6+deb9u2, automatic), '
        'binutils:amd64 (2.28-5, automatic), '
        'cpp:amd64 (4:6.3.0-4, automatic), '
        'python-dbus:amd64 (1.2.4-1+b1, automatic), '
        'libitm1:amd64 (6.3.0-18+deb9u1, automatic), '
        'libpython-all-dev:amd64 (2.7.13-2, automatic), '
        'g++:amd64 (4:6.3.0-4, automatic), '
        'perl-modules-5.24:amd64 (5.24.1-3+deb9u5, automatic), '
        'libpython2.7:amd64 (2.7.13-2+deb9u3, automatic), '
        'python3-pip:amd64 (9.0.1-2+deb9u1), '
        'python3-keyring:amd64 (10.1-1, automatic), '
        'python3-wheel:amd64 (0.29.0-2, automatic), '
        'libpython3.5:amd64 (3.5.3-1+deb9u1, automatic), '
        'gcc:amd64 (4:6.3.0-4, automatic), '
        'git:amd64 (1:2.11.0-3+deb9u4), '
        'python3-idna:amd64 (2.2-1, automatic), '
        'libpython2.7-dev:amd64 (2.7.13-2+deb9u3, automatic), '
        'libcilkrts5:amd64 (6.3.0-18+deb9u1, automatic), '
        'python3-six:amd64 (1.10.0-3, automatic), '
        'libasan3:amd64 (6.3.0-18+deb9u1, automatic), '
        'libquadmath0:amd64 (6.3.0-18+deb9u1, automatic), '
        'python3.5-dev:amd64 (3.5.3-1+deb9u1, automatic), '
        'joe:amd64 (4.4-1), '
        'libisl15:amd64 (0.18-1, automatic), '
        'python-enum34:amd64 (1.1.6-1, automatic), '
        'build-essential:amd64 (12.3, automatic), '
        'libfile-fcntllock-perl:amd64 (0.22-3+b2, automatic), '
        'python3-xdg:amd64 (0.25-4, automatic), '
        'libperl5.24:amd64 (5.24.1-3+deb9u5, automatic), '
        'python-cryptography:amd64 (1.7.1-3+deb9u1, automatic), '
        'python-crypto:amd64 (2.6.1-7, automatic), '
        'xml-core:amd64 (0.17, automatic), '
        'python-keyrings.alt:amd64 (1.3-1, automatic), '
        'python-cffi-backend:amd64 (1.9.1-2, automatic), '
        'python-ipaddress:amd64 (1.0.17-1, automatic), '
        'libtsan0:amd64 (6.3.0-18+deb9u1, automatic), '
        'libgcc-6-dev:amd64 (6.3.0-18+deb9u1, automatic), '
        'python3-cryptography:amd64 (1.7.1-3+deb9u1, automatic), '
        'libubsan0:amd64 (6.3.0-18+deb9u1, automatic), '
        'gir1.2-glib-2.0:amd64 (1.50.0-1+b1, automatic), '
        'g++-6:amd64 (6.3.0-18+deb9u1, automatic), '
        'libpython-dev:amd64 (2.7.13-2, automatic), '
        'python3-cffi-backend:amd64 (1.9.1-2, automatic), '
        'make:amd64 (4.1-9.1, automatic), '
        'fakeroot:amd64 (1.21-3.1, automatic), '
        'gcc-6:amd64 (6.3.0-18+deb9u1, automatic), '
        'python-all:amd64 (2.7.13-2, automatic), '
        'liblsan0:amd64 (6.3.0-18+deb9u1, automatic), '
        'shared-mime-info:amd64 (1.8-1+deb9u1, automatic), '
        'libgomp1:amd64 (6.3.0-18+deb9u1, automatic), '
        'libpython3.5-dev:amd64 (3.5.3-1+deb9u1, automatic), '
        'libdbus-glib-1-2:amd64 (0.108-2, automatic), '
        'libutempter0:amd64 (1.1.6-3, automatic), '
        'python-secretstorage:amd64 (2.3.1-2, automatic), '
        'python-dev:amd64 (2.7.13-2), '
        'python-pyasn1:amd64 (0.1.9-2, automatic), '
        'python-setuptools:amd64 (33.1.1-1, automatic), '
        'python-keyring:amd64 (10.1-1, automatic), '
        'manpages-dev:amd64 (4.10-2, automatic), '
        'bzip2:amd64 (1.0.6-8.1, automatic), '
        'python3-dbus:amd64 (1.2.4-1+b1, automatic), '
        'libevent-2.0-5:amd64 (2.0.21-stable-3, automatic), '
        'libglib2.0-data:amd64 (2.50.3-2, automatic), '
        'python3-keyrings.alt:amd64 (1.3-1, automatic), '
        'libc-dev-bin:amd64 (2.24-11+deb9u4, automatic), '
        'libxml2:amd64 (2.9.4+dfsg1-2.2+deb9u2, automatic), '
        'python3-gi:amd64 (3.22.0-2, automatic), '
        'python3-crypto:amd64 (2.6.1-7, automatic), '
        'perl:amd64 (5.24.1-3+deb9u5, automatic), '
        'rsync:amd64 (3.1.2-1+deb9u2, automatic), '
        'tmux:amd64 (2.3-4), '
        'python-idna:amd64 (2.2-1, automatic), '
        'python-wheel:amd64 (0.29.0-2, automatic), '
        'libpython3-dev:amd64 (3.5.3-1, automatic), '
        'libatomic1:amd64 (6.3.0-18+deb9u1, automatic), '
        'libcc1-0:amd64 (6.3.0-18+deb9u1, automatic), '
        'libdpkg-perl:amd64 (1.18.25, automatic), '
        'patch:amd64 (2.7.5-1+deb9u1, automatic), '
        'python3-secretstorage:amd64 (2.3.1-2, automatic), '
        'libalgorithm-diff-xs-perl:amd64 (0.04-4+b2, automatic), '
        'python-pip-whl:amd64 (9.0.1-2+deb9u1, automatic), '
        'libglib2.0-0:amd64 (2.50.3-2, automatic), '
        'python-pip:amd64 (9.0.1-2+deb9u1), '
        'libcurl3-gnutls:amd64 (7.52.1-5+deb9u9, automatic), '
        'less:amd64 (481-2.1, automatic), '
        'python3-setuptools:amd64 (33.1.1-1, automatic), '
        'dpkg-dev:amd64 (1.18.25, automatic), '
        'python-all-dev:amd64 (2.7.13-2, automatic), '
        'python3-pyasn1:amd64 (0.1.9-2, automatic), '
        'libstdc++-6-dev:amd64 (6.3.0-18+deb9u1, automatic), '
        'liberror-perl:amd64 (0.17024-1, automatic) '
        '[Commandline: apt-get -y install python-pip python3-pip python-dev '
        'python3-dev git tmux screen joe]')
    expected_short_message = (
        'Install: libmpc3:amd64 (1.0.3-1+b2, automatic), '
        'manpages:amd64 (4.10-2, autom...')
    self._TestGetMessageStrings(
        event_data, expected_message, expected_short_message)

    event = events[4]
    event_data = self._GetEventDataOfEvent(storage_writer, event)

    expected_message = (
        'Install: containerd.io:amd64 (1.2.6-3), '
        'linux-headers-4.9.0-9-common:amd64 (4.9.168-1+deb9u3, automatic), '
        'aufs-tools:amd64 (1:4.1+20161219-1, automatic), '
        'aufs-dkms:amd64 (4.9+20161219-1, automatic), '
        'cgroupfs-mount:amd64 (1.3, automatic), '
        'linux-compiler-gcc-6-x86:amd64 (4.9.168-1+deb9u3, automatic), '
        'dkms:amd64 (2.3-2, automatic), libltdl7:amd64 (2.4.6-2, automatic), '
        'linux-headers-amd64:amd64 (4.9+80+deb9u7, automatic), '
        'linux-kbuild-4.9:amd64 (4.9.168-1+deb9u3, automatic), '
        'linux-headers-4.9.0-9-amd64:amd64 (4.9.168-1+deb9u3, automatic), '
        'pigz:amd64 (2.3.4-1, automatic), '
        'docker-ce:amd64 (5:18.09.7~3-0~debian-stretch), '
        'docker-ce-cli:amd64 (5:18.09.7~3-0~debian-stretch) '
        '[Commandline: apt-get install -y docker-ce docker-ce-cli containerd.io'
        '] [Error: Sub-process /usr/bin/dpkg returned an error code (1)]')
    expected_short_message = (
        'Install: containerd.io:amd64 (1.2.6-3), '
        'linux-headers-4.9.0-9-common:amd64 (4...')
    self._TestGetMessageStrings(
        event_data, expected_message, expected_short_message)

    event = events[6]
    event_data = self._GetEventDataOfEvent(storage_writer, event)
    expected_message = (
        'Remove: volatility:amd64 (2.6-1), forensics-all:amd64 (1.5) '
        '[Commandline: apt-get remove volatility] '
        '[Requested-By: jxs (1005)]')
    expected_short_message = (
        'Remove: volatility:amd64 (2.6-1), forensics-all:amd64 (1.5)')
    self._TestGetMessageStrings(
        event_data, expected_message, expected_short_message)

    event = events[9]
    event_data = self._GetEventDataOfEvent(storage_writer, event)

    self.CheckTimestamp(event.timestamp, '2019-07-12 04:19:26.000000')

    expected_message = (
        'Remove: python-distorm3:amd64 (3.3.4-2), '
        'python-imaging:amd64 (4.0.0-4), python-py:amd64 (1.4.32-3), '
        'python-openpyxl:amd64 (2.3.0-3), libdistorm3-3:amd64 (3.3.4-2), '
        'python-jdcal:amd64 (1.0-1.2~deb9u1) [Commandline: apt-get autoremove] '
        '[Requested-By: jxs (1005)]')
    expected_short_message = (
        'Remove: python-distorm3:amd64 (3.3.4-2), '
        'python-imaging:amd64 (4.0.0-4), pyth...')
    self._TestGetMessageStrings(
        event_data, expected_message, expected_short_message)

  def testParseInvalidLog(self):
    """Tests the Parse function on a non APT History log."""
    parser = apt_history.APTHistoryLogParser()
    with self.assertRaises(errors.UnableToParseFile):
      self._ParseFile(['setupapi.dev.log'], parser)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The event extraction worker."""

from __future__ import unicode_literals

import copy
import os
import re
import time

from dfvfs.analyzer import analyzer
from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.lib import errors as dfvfs_errors
from dfvfs.path import factory as path_spec_factory
from dfvfs.resolver import resolver as path_spec_resolver

from plaso.analyzers import hashing_analyzer
from plaso.analyzers import manager as analyzers_manager
from plaso.containers import event_sources
from plaso.engine import extractors
from plaso.engine import logger
from plaso.lib import definitions
from plaso.lib import errors


class EventExtractionWorker(object):
  """Event extraction worker.

  The event extraction worker determines which parsers are suitable for parsing
  a particular file entry or data stream. The parsers extract relevant data from
  file system and or file content data. All extracted data is passed to the
  parser mediator for further processing.

  Attributes:
    last_activity_timestamp (int): timestamp received that indicates the last
        time activity was observed.
    processing_status (str): human readable status indication such as:
        'Extracting', 'Hashing'.
  """

  # TSK metadata files that need special handling.
  _METADATA_FILE_LOCATIONS_TSK = frozenset([
      # NTFS
      '/$AttrDef',
      '/$BadClus',
      '/$Bitmap',
      '/$Boot',
      '/$Extend/$ObjId',
      '/$Extend/$Quota',
      '/$Extend/$Reparse',
      '/$Extend/$RmMetadata/$Repair',
      '/$Extend/$RmMetadata/$TxfLog/$Tops',
      '/$Extend/$UsnJrnl',
      '/$LogFile',
      '/$MFT',
      '/$MFTMirr',
      '/$Secure',
      '/$UpCase',
      '/$Volume',
      # HFS+/HFSX
      '/$ExtentsFile',
      '/$CatalogFile',
      '/$BadBlockFile',
      '/$AllocationFile',
      '/$AttributesFile',
  ])

  # TODO: make this filtering solution more generic. Also see:
  # https://github.com/log2timeline/plaso/issues/467
  _CHROME_CACHE_DATA_FILE_RE = re.compile(r'^[fF]_[0-9a-fA-F]{6}$')
  _FIREFOX_CACHE_DATA_FILE_RE = re.compile(r'^[0-9a-fA-F]{5}[dm][0-9]{2}$')
  _FIREFOX_CACHE2_DATA_FILE_RE = re.compile(r'^[0-9a-fA-F]{40}$')

  _TYPES_WITH_ROOT_METADATA = frozenset([
      dfvfs_definitions.TYPE_INDICATOR_GZIP])

  def __init__(self, parser_filter_expression=None):
    """Initializes an event extraction worker.

    Args:
      parser_filter_expression (Optional[str]): parser filter expression,
          where None represents all parsers and plugins.

          A parser filter expression is a comma separated value string that
          denotes which parsers and plugins should be used. See
          filters/parser_filter.py for details of the expression syntax.

          This function does not support presets, and requires a parser
          filter expression where presets have been expanded.
    """
    super(EventExtractionWorker, self).__init__()
    self._abort = False
    self._analyzers = []
    self._event_extractor = extractors.EventExtractor(
        parser_filter_expression=parser_filter_expression)
    self._hasher_file_size_limit = None
    self._path_spec_extractor = extractors.PathSpecExtractor()
    self._process_archives = None
    self._process_compressed_streams = None
    self._processing_profiler = None

    self.last_activity_timestamp = 0.0
    self.processing_status = definitions.STATUS_INDICATOR_IDLE

  def _AnalyzeDataStream(self, mediator, file_entry, data_stream_name):
    """Analyzes the contents of a specific data stream of a file entry.

    The results of the analyzers are set in the parser mediator as attributes
    that are added to produced event objects. Note that some file systems
    allow directories to have data streams, such as NTFS.

    Args:
      mediator (ParserMediator): mediates the interactions between
          parsers and other components, such as storage and abort signals.
      file_entry (dfvfs.FileEntry): file entry whose data stream is to be
          analyzed.
      data_stream_name (str): name of the data stream.

    Raises:
      RuntimeError: if the file-like object cannot be retrieved from
          the file entry.
    """
    display_name = mediator.GetDisplayName()
    logger.debug('[AnalyzeDataStream] analyzing file: {0:s}'.format(
        display_name))

    if self._processing_profiler:
      self._processing_profiler.StartTiming('analyzing')

    try:
      file_object = file_entry.GetFileObject(data_stream_name=data_stream_name)
      if not file_object:
        raise RuntimeError((
            'Unable to retrieve file-like object for file entry: '
            '{0:s}.').format(display_name))

      try:
        self._AnalyzeFileObject(mediator, file_object)
      finally:
        file_object.close()

    finally:
      if self._processing_profiler:
        self._processing_profiler.StopTiming('analyzing')

    logger.debug(
        '[AnalyzeDataStream] completed analyzing file: {0:s}'.format(
            display_name))

  def _AnalyzeFileObject(self, mediator, file_object):
    """Processes a file-like object with analyzers.

    Args:
      mediator (ParserMediator): mediates the interactions between
          parsers and other components, such as storage and abort signals.
      file_object (dfvfs.FileIO): file-like object to process.
    """
    maximum_read_size = max([
        analyzer_object.SIZE_LIMIT for analyzer_object in self._analyzers])

    hashers_only = True
    for analyzer_object in self._analyzers:
      if not isinstance(analyzer_object, hashing_analyzer.HashingAnalyzer):
        hashers_only = False
        break

    file_size = file_object.get_size()

    if (hashers_only and self._hasher_file_size_limit and
        file_size > self._hasher_file_size_limit):
      return

    file_object.seek(0, os.SEEK_SET)

    data = file_object.read(maximum_read_size)
    while data:
      if self._abort:
        break

      for analyzer_object in self._analyzers:
        if self._abort:
          break

        if (not analyzer_object.INCREMENTAL_ANALYZER and
            file_size > analyzer_object.SIZE_LIMIT):
          continue

        if (isinstance(analyzer_object, hashing_analyzer.HashingAnalyzer) and
            self._hasher_file_size_limit and
            file_size > self._hasher_file_size_limit):
          continue

        self.processing_status = analyzer_object.PROCESSING_STATUS_HINT

        analyzer_object.Analyze(data)

        self.last_activity_timestamp = time.time()

      data = file_object.read(maximum_read_size)

    display_name = mediator.GetDisplayName()
    for analyzer_object in self._analyzers:
      if self._abort:
        break

      for result in analyzer_object.GetResults():
        logger.debug((
            '[AnalyzeFileObject] attribute {0:s}:{1:s} calculated for '
            'file: {2:s}.').format(
                result.attribute_name, result.attribute_value, display_name))

        mediator.AddEventAttribute(
            result.attribute_name, result.attribute_value)

      analyzer_object.Reset()

    self.processing_status = definitions.STATUS_INDICATOR_RUNNING

  def _CanSkipDataStream(self, file_entry, data_stream):
    """Determines if analysis and extraction of a data stream can be skipped.

    This is used to prevent Plaso trying to run analyzers or extract content
    from a pipe or socket it encounters while processing a mounted filesystem.

    Args:
      file_entry (dfvfs.FileEntry): file entry to consider for skipping.
      data_stream (dfvfs.DataStream): data stream to consider for skipping.

    Returns:
      bool: True if the data stream can be skipped.
    """
    if file_entry.IsFile():
      return False

    if data_stream.IsDefault():
      return True

    return False

  def _CanSkipContentExtraction(self, file_entry):
    """Determines if content extraction of a file entry can be skipped.

    Args:
      file_entry (dfvfs.FileEntry): file entry of which to determine content
          extraction can be skipped.

    Returns:
      bool: True if content extraction can be skipped.
    """
    # TODO: make this filtering solution more generic. Also see:
    # https://github.com/log2timeline/plaso/issues/467
    location = getattr(file_entry.path_spec, 'location', None)
    if not location:
      return False

    data_stream_name = getattr(file_entry.path_spec, 'data_stream', None)
    if data_stream_name:
      return False

    file_system = file_entry.GetFileSystem()

    path_segments = file_system.SplitPath(location)
    if not path_segments:
      return False

    if self._CHROME_CACHE_DATA_FILE_RE.match(path_segments[-1]):
      location_segments = path_segments[:-1]
      location_segments.append('index')
      location = file_system.JoinPath(location_segments)
      index_path_spec = path_spec_factory.Factory.NewPathSpec(
          file_entry.type_indicator, location=location,
          parent=file_entry.path_spec.parent)

      if file_system.FileEntryExistsByPathSpec(index_path_spec):
        # TODO: improve this check if "index" is a Chrome Cache index file.
        return True

    elif self._FIREFOX_CACHE_DATA_FILE_RE.match(path_segments[-1]):
      location_segments = path_segments[:-4]
      location_segments.append('_CACHE_MAP_')
      location = file_system.JoinPath(location_segments)
      cache_map_path_spec = path_spec_factory.Factory.NewPathSpec(
          file_entry.type_indicator, location=location,
          parent=file_entry.path_spec.parent)

      if file_system.FileEntryExistsByPathSpec(cache_map_path_spec):
        # TODO: improve this check if "_CACHE_MAP_" is a Firefox Cache
        # version 1 cache map file.
        return True

    elif self._FIREFOX_CACHE2_DATA_FILE_RE.match(path_segments[-1]):
      location_segments = path_segments[:-2]
      location_segments.append('index')
      location = file_system.JoinPath(location_segments)
      index_path_spec = path_spec_factory.Factory.NewPathSpec(
          file_entry.type_indicator, location=location,
          parent=file_entry.path_spec.parent)

      if file_system.FileEntryExistsByPathSpec(index_path_spec):
        # TODO: improve this check if "index" is a Firefox Cache version 2
        # index file.
        return True

    elif len(path_segments) == 1 and path_segments[0].lower() in (
        'hiberfil.sys', 'pagefile.sys', 'swapfile.sys'):
      return True

    return False

  def _ExtractContentFromDataStream(
      self, mediator, file_entry, data_stream_name):
    """Extracts content from a data stream.

    Args:
      mediator (ParserMediator): mediates the interactions between
          parsers and other components, such as storage and abort signals.
      file_entry (dfvfs.FileEntry): file entry to extract its content.
      data_stream_name (str): name of the data stream whose content is to be
          extracted.
    """
    self.processing_status = definitions.STATUS_INDICATOR_EXTRACTING

    if self._processing_profiler:
      self._processing_profiler.StartTiming('extracting')

    self._event_extractor.ParseDataStream(
        mediator, file_entry, data_stream_name)

    if self._processing_profiler:
      self._processing_profiler.StopTiming('extracting')

    self.processing_status = definitions.STATUS_INDICATOR_RUNNING

    self.last_activity_timestamp = time.time()

  def _ExtractMetadataFromFileEntry(self, mediator, file_entry, data_stream):
    """Extracts metadata from a file entry.

    Args:
      mediator (ParserMediator): mediates the interactions between
          parsers and other components, such as storage and abort signals.
      file_entry (dfvfs.FileEntry): file entry to extract metadata from.
      data_stream (dfvfs.DataStream): data stream or None if the file entry
          has no data stream.
    """
    # Do not extract metadata from the root file entry when it is virtual.
    if file_entry.IsRoot() and file_entry.type_indicator not in (
        self._TYPES_WITH_ROOT_METADATA):
      return

    # We always want to extract the file entry metadata but we only want
    # to parse it once per file entry, so we only use it if we are
    # processing the default data stream of regular files.
    if data_stream and not data_stream.IsDefault():
      return

    display_name = mediator.GetDisplayName()
    logger.debug(
        '[ExtractMetadataFromFileEntry] processing file entry: {0:s}'.format(
            display_name))

    self.processing_status = definitions.STATUS_INDICATOR_EXTRACTING

    if self._processing_profiler:
      self._processing_profiler.StartTiming('extracting')

    self._event_extractor.ParseFileEntryMetadata(mediator, file_entry)

    if self._processing_profiler:
      self._processing_profiler.StopTiming('extracting')

    self.processing_status = definitions.STATUS_INDICATOR_RUNNING

  def _GetArchiveTypes(self, mediator, path_spec):
    """Determines if a data stream contains an archive such as: TAR or ZIP.

    Args:
      mediator (ParserMediator): mediates the interactions between
          parsers and other components, such as storage and abort signals.
      path_spec (dfvfs.PathSpec): path specification of the data stream.

    Returns:
      list[str]: dfVFS archive type indicators found in the data stream.
    """
    try:
      type_indicators = analyzer.Analyzer.GetArchiveTypeIndicators(
          path_spec, resolver_context=mediator.resolver_context)
    except IOError as exception:
      type_indicators = []

      warning_message = (
          'analyzer failed to determine archive type indicators '
          'with error: {0!s}').format(exception)
      mediator.ProduceExtractionWarning(warning_message, path_spec=path_spec)

    return type_indicators

  def _GetCompressedStreamTypes(self, mediator, path_spec):
    """Determines if a data stream contains a compressed stream such as: gzip.

    Args:
      mediator (ParserMediator): mediates the interactions between
          parsers and other components, such as storage and abort signals.
      path_spec (dfvfs.PathSpec): path specification of the data stream.

    Returns:
      list[str]: dfVFS compressed stream type indicators found in
          the data stream.
    """
    try:
      type_indicators = analyzer.Analyzer.GetCompressedStreamTypeIndicators(
          path_spec, resolver_context=mediator.resolver_context)
    except IOError as exception:
      type_indicators = []

      warning_message = (
          'analyzer failed to determine compressed stream type indicators '
          'with error: {0!s}').format(exception)
      mediator.ProduceExtractionWarning(warning_message, path_spec=path_spec)

    return type_indicators

  def _IsMetadataFile(self, file_entry):
    """Determines if the file entry is a metadata file.

    Args:
      file_entry (dfvfs.FileEntry): a file entry object.

    Returns:
      bool: True if the file entry is a metadata file.
    """
    if (file_entry.type_indicator == dfvfs_definitions.TYPE_INDICATOR_TSK and
        file_entry.path_spec.location in self._METADATA_FILE_LOCATIONS_TSK):
      return True

    return False

  def _ProcessArchiveTypes(self, mediator, path_spec, type_indicators):
    """Processes a data stream containing archive types such as: TAR or ZIP.

    Args:
      mediator (ParserMediator): mediates the interactions between
          parsers and other components, such as storage and abort signals.
      path_spec (dfvfs.PathSpec): path specification.
      type_indicators(list[str]): dfVFS archive type indicators found in
          the data stream.
    """
    number_of_type_indicators = len(type_indicators)
    if number_of_type_indicators == 0:
      return

    self.processing_status = definitions.STATUS_INDICATOR_COLLECTING

    if number_of_type_indicators > 1:
      display_name = mediator.GetDisplayName()
      logger.debug((
          'Found multiple format type indicators: {0:s} for '
          'archive file: {1:s}').format(type_indicators, display_name))

    for type_indicator in type_indicators:
      if type_indicator == dfvfs_definitions.TYPE_INDICATOR_TAR:
        archive_path_spec = path_spec_factory.Factory.NewPathSpec(
            dfvfs_definitions.TYPE_INDICATOR_TAR, location='/',
            parent=path_spec)

      elif type_indicator == dfvfs_definitions.TYPE_INDICATOR_ZIP:
        archive_path_spec = path_spec_factory.Factory.NewPathSpec(
            dfvfs_definitions.TYPE_INDICATOR_ZIP, location='/',
            parent=path_spec)

      else:
        archive_path_spec = None

        warning_message = (
            'unsupported archive format type indicator: {0:s}').format(
                type_indicator)
        mediator.ProduceExtractionWarning(
            warning_message, path_spec=path_spec)

      if archive_path_spec:
        try:
          path_spec_generator = self._path_spec_extractor.ExtractPathSpecs(
              [archive_path_spec], resolver_context=mediator.resolver_context)

          for generated_path_spec in path_spec_generator:
            if self._abort:
              break

            event_source = event_sources.FileEntryEventSource(
                path_spec=generated_path_spec)
            event_source.file_entry_type = (
                dfvfs_definitions.FILE_ENTRY_TYPE_FILE)
            mediator.ProduceEventSource(event_source)

            self.last_activity_timestamp = time.time()

        except (IOError, errors.MaximumRecursionDepth) as exception:
          warning_message = (
              'unable to process archive file with error: {0!s}').format(
                  exception)
          mediator.ProduceExtractionWarning(
              warning_message, path_spec=generated_path_spec)

  def _ProcessCompressedStreamTypes(self, mediator, path_spec, type_indicators):
    """Processes a data stream containing compressed stream types such as: bz2.

    Args:
      mediator (ParserMediator): mediates the interactions between
          parsers and other components, such as storage and abort signals.
      path_spec (dfvfs.PathSpec): path specification.
      type_indicators(list[str]): dfVFS archive type indicators found in
          the data stream.
    """
    number_of_type_indicators = len(type_indicators)
    if number_of_type_indicators == 0:
      return

    self.processing_status = definitions.STATUS_INDICATOR_COLLECTING

    if number_of_type_indicators > 1:
      display_name = mediator.GetDisplayName()
      logger.debug((
          'Found multiple format type indicators: {0:s} for '
          'compressed stream file: {1:s}').format(
              type_indicators, display_name))

    for type_indicator in type_indicators:
      if type_indicator == dfvfs_definitions.TYPE_INDICATOR_BZIP2:
        compressed_stream_path_spec = path_spec_factory.Factory.NewPathSpec(
            dfvfs_definitions.TYPE_INDICATOR_COMPRESSED_STREAM,
            compression_method=dfvfs_definitions.COMPRESSION_METHOD_BZIP2,
            parent=path_spec)

      elif type_indicator == dfvfs_definitions.TYPE_INDICATOR_GZIP:
        compressed_stream_path_spec = path_spec_factory.Factory.NewPathSpec(
            dfvfs_definitions.TYPE_INDICATOR_GZIP, parent=path_spec)

      else:
        compressed_stream_path_spec = None

        warning_message = (
            'unsupported compressed stream format type indicators: '
            '{0:s}').format(type_indicator)
        mediator.ProduceExtractionWarning(
            warning_message, path_spec=path_spec)

      if compressed_stream_path_spec:
        event_source = event_sources.FileEntryEventSource(
            path_spec=compressed_stream_path_spec)
        event_source.file_entry_type = dfvfs_definitions.FILE_ENTRY_TYPE_FILE
        mediator.ProduceEventSource(event_source)

        self.last_activity_timestamp = time.time()

  def _ProcessDirectory(self, mediator, file_entry):
    """Processes a directory file entry.

    Args:
      mediator (ParserMediator): mediates the interactions between
          parsers and other components, such as storage and abort signals.
      file_entry (dfvfs.FileEntry): file entry of the directory.
    """
    self.processing_status = definitions.STATUS_INDICATOR_COLLECTING

    if self._processing_profiler:
      self._processing_profiler.StartTiming('collecting')

    for sub_file_entry in file_entry.sub_file_entries:
      if self._abort:
        break

      try:
        if not sub_file_entry.IsAllocated():
          continue

      except dfvfs_errors.BackEndError as exception:
        warning_message = (
            'unable to process directory entry: {0:s} with error: '
            '{1!s}').format(sub_file_entry.name, exception)
        mediator.ProduceExtractionWarning(
            warning_message, path_spec=file_entry.path_spec)
        continue

      # For TSK-based file entries only, ignore the virtual /$OrphanFiles
      # directory.
      if sub_file_entry.type_indicator == dfvfs_definitions.TYPE_INDICATOR_TSK:
        if file_entry.IsRoot() and sub_file_entry.name == '$OrphanFiles':
          continue

      event_source = event_sources.FileEntryEventSource(
          path_spec=sub_file_entry.path_spec)

      # TODO: move this into a dfVFS file entry property.
      stat_object = sub_file_entry.GetStat()
      if stat_object:
        event_source.file_entry_type = stat_object.type

      mediator.ProduceEventSource(event_source)

      self.last_activity_timestamp = time.time()

    if self._processing_profiler:
      self._processing_profiler.StopTiming('collecting')

    self.processing_status = definitions.STATUS_INDICATOR_RUNNING

  def _ProcessFileEntry(self, mediator, file_entry):
    """Processes a file entry.

    Args:
      mediator (ParserMediator): mediates the interactions between
          parsers and other components, such as storage and abort signals.
      file_entry (dfvfs.FileEntry): file entry.
    """
    display_name = mediator.GetDisplayName()
    logger.debug(
        '[ProcessFileEntry] processing file entry: {0:s}'.format(display_name))

    reference_count = mediator.resolver_context.GetFileObjectReferenceCount(
        file_entry.path_spec)

    try:
      if self._IsMetadataFile(file_entry):
        self._ProcessMetadataFile(mediator, file_entry)

      else:
        file_entry_processed = False
        for data_stream in file_entry.data_streams:
          if self._abort:
            break

          if self._CanSkipDataStream(file_entry, data_stream):
            logger.debug((
                '[ProcessFileEntry] Skipping datastream {0:s} for {1:s}: '
                '{2:s}').format(
                    data_stream.name, file_entry.type_indicator, display_name))
            continue

          self._ProcessFileEntryDataStream(mediator, file_entry, data_stream)

          file_entry_processed = True

        if not file_entry_processed:
          # For when the file entry does not contain a data stream.
          self._ProcessFileEntryDataStream(mediator, file_entry, None)

    finally:
      new_reference_count = (
          mediator.resolver_context.GetFileObjectReferenceCount(
              file_entry.path_spec))
      if reference_count != new_reference_count:
        # Clean up after parsers that do not call close explicitly.
        if mediator.resolver_context.ForceRemoveFileObject(
            file_entry.path_spec):
          logger.warning(
              'File-object not explicitly closed for file: {0:s}'.format(
                  display_name))

    logger.debug(
        '[ProcessFileEntry] done processing file entry: {0:s}'.format(
            display_name))

  def _ProcessFileEntryDataStream(self, mediator, file_entry, data_stream):
    """Processes a specific data stream of a file entry.

    Args:
      mediator (ParserMediator): mediates the interactions between
          parsers and other components, such as storage and abort signals.
      file_entry (dfvfs.FileEntry): file entry containing the data stream.
      data_stream (dfvfs.DataStream): data stream or None if the file entry
          has no data stream.
    """
    display_name = mediator.GetDisplayName()
    data_stream_name = getattr(data_stream, 'name', '') or ''
    logger.debug((
        '[ProcessFileEntryDataStream] processing data stream: "{0:s}" of '
        'file entry: {1:s}').format(data_stream_name, display_name))

    mediator.ClearEventAttributes()

    if data_stream and self._analyzers:
      # Since AnalyzeDataStream generates event attributes it needs to be
      # called before producing events.
      self._AnalyzeDataStream(mediator, file_entry, data_stream.name)

    self._ExtractMetadataFromFileEntry(mediator, file_entry, data_stream)

    # Not every file entry has a data stream. In such cases we want to
    # extract the metadata only.
    if not data_stream:
      return

    # Determine if the content of the file entry should not be extracted.
    skip_content_extraction = self._CanSkipContentExtraction(file_entry)
    if skip_content_extraction:
      display_name = mediator.GetDisplayName()
      logger.debug(
          'Skipping content extraction of: {0:s}'.format(display_name))
      self.processing_status = definitions.STATUS_INDICATOR_IDLE
      return

    path_spec = copy.deepcopy(file_entry.path_spec)
    if data_stream and not data_stream.IsDefault():
      path_spec.data_stream = data_stream.name

    archive_types = []
    compressed_stream_types = []

    if self._process_compressed_streams:
      compressed_stream_types = self._GetCompressedStreamTypes(
          mediator, path_spec)

    if not compressed_stream_types:
      archive_types = self._GetArchiveTypes(mediator, path_spec)

    if archive_types:
      if self._process_archives:
        self._ProcessArchiveTypes(mediator, path_spec, archive_types)

      if dfvfs_definitions.TYPE_INDICATOR_ZIP in archive_types:
        # ZIP files are the base of certain file formats like docx.
        self._ExtractContentFromDataStream(
            mediator, file_entry, data_stream.name)

    elif compressed_stream_types:
      self._ProcessCompressedStreamTypes(
          mediator, path_spec, compressed_stream_types)

    else:
      self._ExtractContentFromDataStream(
          mediator, file_entry, data_stream.name)

  def _ProcessMetadataFile(self, mediator, file_entry):
    """Processes a metadata file.

    Args:
      mediator (ParserMediator): mediates the interactions between
          parsers and other components, such as storage and abort signals.
      file_entry (dfvfs.FileEntry): file entry of the metadata file.
    """
    self.processing_status = definitions.STATUS_INDICATOR_EXTRACTING

    self._event_extractor.ParseFileEntryMetadata(mediator, file_entry)
    for data_stream in file_entry.data_streams:
      if self._abort:
        break
      self.last_activity_timestamp = time.time()

      self._event_extractor.ParseMetadataFile(
          mediator, file_entry, data_stream.name)

  def _SetHashers(self, hasher_names_string):
    """Sets the hasher names.

    Args:
      hasher_names_string (str): comma separated names of the hashers
          to enable, where 'none' disables the hashing analyzer.
    """
    if not hasher_names_string or hasher_names_string == 'none':
      return

    analyzer_object = analyzers_manager.AnalyzersManager.GetAnalyzerInstance(
        'hashing')
    analyzer_object.SetHasherNames(hasher_names_string)
    self._analyzers.append(analyzer_object)

  def _SetYaraRules(self, yara_rules_string):
    """Sets the Yara rules.

    Args:
      yara_rules_string (str): unparsed Yara rule definitions.
    """
    if not yara_rules_string:
      return

    analyzer_object = analyzers_manager.AnalyzersManager.GetAnalyzerInstance(
        'yara')
    analyzer_object.SetRules(yara_rules_string)
    self._analyzers.append(analyzer_object)

  def GetAnalyzerNames(self):
    """Gets the names of the active analyzers.

    Returns:
      list[str]: names of active analyzers.
    """
    return [analyzer_instance.NAME for analyzer_instance in self._analyzers]

  def ProcessPathSpec(self, mediator, path_spec, excluded_find_specs=None):
    """Processes a path specification.

    Args:
      mediator (ParserMediator): mediates the interactions between
          parsers and other components, such as storage and abort signals.
      path_spec (dfvfs.PathSpec): path specification.
      excluded_find_specs (Optional[list[dfvfs.FindSpec]]): find specifications
         that are excluded from processing.
    """
    self.last_activity_timestamp = time.time()
    self.processing_status = definitions.STATUS_INDICATOR_RUNNING

    file_entry = path_spec_resolver.Resolver.OpenFileEntry(
        path_spec, resolver_context=mediator.resolver_context)

    if file_entry is None:
      display_name = mediator.GetDisplayNameForPathSpec(path_spec)
      logger.warning(
          'Unable to open file entry with path spec: {0:s}'.format(
              display_name))
      self.processing_status = definitions.STATUS_INDICATOR_IDLE
      return

    for find_spec in excluded_find_specs or []:
      if find_spec.Matches(file_entry) == (True, True):
        logger.info('Skipped: {0:s} because of exclusion filter.'.format(
            file_entry.path_spec.location))
        return

    mediator.SetFileEntry(file_entry)

    try:
      if file_entry.IsDirectory():
        self._ProcessDirectory(mediator, file_entry)
      self._ProcessFileEntry(mediator, file_entry)

    finally:
      mediator.ResetFileEntry()

      self.last_activity_timestamp = time.time()
      self.processing_status = definitions.STATUS_INDICATOR_IDLE

  # TODO: move the functionality of this method into the constructor.
  def SetExtractionConfiguration(self, configuration):
    """Sets the extraction configuration settings.

    Args:
      configuration (ExtractionConfiguration): extraction configuration.
    """
    self._hasher_file_size_limit = configuration.hasher_file_size_limit
    self._SetHashers(configuration.hasher_names_string)
    self._process_archives = configuration.process_archives
    self._process_compressed_streams = configuration.process_compressed_streams
    self._SetYaraRules(configuration.yara_rules_string)

  def SetProcessingProfiler(self, processing_profiler):
    """Sets the parsers profiler.

    Args:
      processing_profiler (ProcessingProfiler): processing profile.
    """
    self._processing_profiler = processing_profiler

  def SignalAbort(self):
    """Signals the extraction worker to abort."""
    self._abort = True



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the NTFS metadata file parser."""

from __future__ import unicode_literals

import unittest

from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.path import factory as path_spec_factory

from plaso.formatters import file_system  # pylint: disable=unused-import
from plaso.lib import definitions
from plaso.parsers import ntfs

from tests.parsers import test_lib


class NTFSMFTParserTest(test_lib.ParserTestCase):
  """Tests for NTFS $MFT metadata file parser."""

  def testParseFile(self):
    """Tests the Parse function on a stand-alone $MFT file."""
    parser = ntfs.NTFSMFTParser()

    test_file_path = self._GetTestFilePath(['MFT'])
    self._SkipIfPathNotExists(test_file_path)

    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_file_path)

    storage_writer = self._ParseFileByPathSpec(os_path_spec, parser)

    self.assertEqual(storage_writer.number_of_warnings, 0)
    self.assertEqual(storage_writer.number_of_events, 126352)

    events = list(storage_writer.GetEvents())

    # A distributed link tracking event.
    event = events[3684]

    self.CheckTimestamp(event.timestamp, '2007-06-30 12:58:40.500004')
    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_CREATION)

    event_data = self._GetEventDataOfEvent(storage_writer, event)

    expected_message = (
        '9fe44b69-2709-11dc-a06b-db3099beae3c '
        'MAC address: db:30:99:be:ae:3c '
        'Origin: $MFT: 462-1')

    expected_short_message = (
        '9fe44b69-2709-11dc-a06b-db3099beae3c '
        'Origin: $MFT: 462-1')

    self._TestGetMessageStrings(
        event_data, expected_message, expected_short_message)

  def testParseImage(self):
    """Tests the Parse function on a storage media image."""
    parser = ntfs.NTFSMFTParser()

    test_file_path = self._GetTestFilePath(['vsstest.qcow2'])
    self._SkipIfPathNotExists(test_file_path)

    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_file_path)
    qcow_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_QCOW, parent=os_path_spec)
    tsk_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, inode=0, location='/$MFT',
        parent=qcow_path_spec)

    storage_writer = self._ParseFileByPathSpec(tsk_path_spec, parser)

    self.assertEqual(storage_writer.number_of_warnings, 0)
    self.assertEqual(storage_writer.number_of_events, 284)

    events = list(storage_writer.GetEvents())

    # The creation timestamp.
    event = events[0]

    self.CheckTimestamp(event.timestamp, '2013-12-03 06:30:41.807908')
    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_CREATION)

    event_data = self._GetEventDataOfEvent(storage_writer, event)
    # Check that the allocation status is set correctly.
    self.assertIsInstance(event_data.is_allocated, bool)
    self.assertTrue(event_data.is_allocated)

    # The last modification timestamp.
    event = events[1]

    self.CheckTimestamp(event.timestamp, '2013-12-03 06:30:41.807908')
    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_MODIFICATION)

    event_data = self._GetEventDataOfEvent(storage_writer, event)
    # Check that the allocation status is set correctly.
    self.assertIsInstance(event_data.is_allocated, bool)
    self.assertTrue(event_data.is_allocated)

    # The last accessed timestamp.
    event = events[2]

    self.CheckTimestamp(event.timestamp, '2013-12-03 06:30:41.807908')
    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_LAST_ACCESS)

    event_data = self._GetEventDataOfEvent(storage_writer, event)
    # Check that the allocation status is set correctly.
    self.assertIsInstance(event_data.is_allocated, bool)
    self.assertTrue(event_data.is_allocated)

    # The entry modification timestamp.
    event = events[3]

    self.CheckTimestamp(event.timestamp, '2013-12-03 06:30:41.807908')
    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_ENTRY_MODIFICATION)

    event_data = self._GetEventDataOfEvent(storage_writer, event)
    # Check that the allocation status is set correctly.
    self.assertIsInstance(event_data.is_allocated, bool)
    self.assertTrue(event_data.is_allocated)

    expected_message = (
        'TSK:/$MFT '
        'File reference: 0-1 '
        'Attribute name: $STANDARD_INFORMATION')

    expected_short_message = (
        '/$MFT 0-1 $STANDARD_INFORMATION')

    self._TestGetMessageStrings(
        event_data, expected_message, expected_short_message)

    # The creation timestamp.
    event = events[4]

    self.CheckTimestamp(event.timestamp, '2013-12-03 06:30:41.807908')
    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_CREATION)

    event_data = self._GetEventDataOfEvent(storage_writer, event)
    # Check that the allocation status is set correctly.
    self.assertIsInstance(event_data.is_allocated, bool)
    self.assertTrue(event_data.is_allocated)

    expected_message = (
        'TSK:/$MFT '
        'File reference: 0-1 '
        'Attribute name: $FILE_NAME '
        'Name: $MFT '
        'Parent file reference: 5-5')

    expected_short_message = (
        '/$MFT 0-1 $FILE_NAME')

    self._TestGetMessageStrings(
        event_data, expected_message, expected_short_message)

    # Note that the source file is a RAW (VMDK flat) image.
    test_file_path = self._GetTestFilePath(['multi_partition_image.vmdk'])
    self._SkipIfPathNotExists(test_file_path)

    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_file_path)
    p2_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK_PARTITION, location='/p2',
        part_index=3, start_offset=0x00510000, parent=os_path_spec)
    tsk_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, inode=0, location='/$MFT',
        parent=p2_path_spec)

    storage_writer = self._ParseFileByPathSpec(tsk_path_spec, parser)

    self.assertEqual(storage_writer.number_of_warnings, 0)
    self.assertEqual(storage_writer.number_of_events, 184)


class NTFSUsnJrnlParser(test_lib.ParserTestCase):
  """Tests for NTFS $UsnJrnl metadata file parser."""

  def testParseImage(self):
    """Tests the Parse function on a storage media image."""
    parser = ntfs.NTFSUsnJrnlParser()

    test_file_path = self._GetTestFilePath(['usnjrnl.qcow2'])
    self._SkipIfPathNotExists(test_file_path)

    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_file_path)
    qcow_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_QCOW, parent=os_path_spec)
    volume_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK_PARTITION, location='/p1',
        part_index=2, start_offset=0x00007e00, parent=qcow_path_spec)

    # To be able to ignore the sparse data ranges the UsnJrnl parser
    # requires to read directly from the volume.
    storage_writer = self._ParseFileByPathSpec(volume_path_spec, parser)

    self.assertEqual(storage_writer.number_of_warnings, 0)
    self.assertEqual(storage_writer.number_of_events, 19)

    events = list(storage_writer.GetEvents())

    event = events[0]

    self.CheckTimestamp(event.timestamp, '2015-11-30 21:15:27.203125')
    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_ENTRY_MODIFICATION)

    event_data = self._GetEventDataOfEvent(storage_writer, event)

    expected_message = (
        'Nieuw - Tekstdocument.txt '
        'File reference: 30-1 '
        'Parent file reference: 5-5 '
        'Update reason: USN_REASON_FILE_CREATE')

    expected_short_message = (
        'Nieuw - Tekstdocument.txt 30-1 USN_REASON_FILE_CREATE')

    self._TestGetMessageStrings(
        event_data, expected_message, expected_short_message)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The default Windows Registry plugin."""

from __future__ import unicode_literals

from plaso.containers import time_events
from plaso.containers import windows_events
from plaso.lib import definitions
from plaso.parsers import winreg
from plaso.parsers.winreg_plugins import interface


class DefaultPlugin(interface.WindowsRegistryPlugin):
  """Default plugin that extracts minimum information from every registry key.

  The default plugin will parse every registry key that is passed to it and
  extract minimum information, such as a list of available values and if
  possible content of those values. The timestamp used is the timestamp
  when the registry key was last modified.
  """

  NAME = 'winreg_default'
  DESCRIPTION = 'Parser for Registry data.'

  def ExtractEvents(self, parser_mediator, registry_key, **kwargs):
    """Extracts events from a Windows Registry key.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.
    """
    values_dict = self._GetValuesFromKey(registry_key)

    event_data = windows_events.WindowsRegistryEventData()
    event_data.key_path = registry_key.path
    event_data.values = ' '.join([
        '{0:s}: {1!s}'.format(name, value)
        for name, value in sorted(values_dict.items())]) or None

    event = time_events.DateTimeValuesEvent(
        registry_key.last_written_time, definitions.TIME_DESCRIPTION_WRITTEN)
    parser_mediator.ProduceEventWithEventData(event, event_data)


winreg.WinRegistryParser.RegisterPlugin(DefaultPlugin)



# -*- coding: utf-8 -*-
"""The Google Chrome history event formatters."""

from __future__ import unicode_literals

from plaso.formatters import interface
from plaso.formatters import manager
from plaso.lib import errors


class ChromeFileDownloadFormatter(interface.ConditionalEventFormatter):
  """Formatter for a Chrome file download event."""

  DATA_TYPE = 'chrome:history:file_downloaded'

  FORMAT_STRING_PIECES = [
      '{url}',
      '({full_path}).',
      'Received: {received_bytes} bytes',
      'out of: {total_bytes} bytes.']

  FORMAT_STRING_SHORT_PIECES = [
      '{full_path} downloaded',
      '({received_bytes} bytes)']

  SOURCE_LONG = 'Chrome History'
  SOURCE_SHORT = 'WEBHIST'


class ChromePageVisitedFormatter(interface.ConditionalEventFormatter):
  """Formatter for a Chrome page visited event."""

  DATA_TYPE = 'chrome:history:page_visited'

  FORMAT_STRING_PIECES = [
      '{url}',
      '({title})',
      '[count: {typed_count}]',
      'Visit from: {from_visit}',
      'Visit Source: [{visit_source}]',
      'Type: [{page_transition}]',
      '{extra}']

  FORMAT_STRING_SHORT_PIECES = [
      '{url}',
      '({title})']

  SOURCE_LONG = 'Chrome History'
  SOURCE_SHORT = 'WEBHIST'

  _UNKNOWN_PAGE_TRANSITION = ('UNKNOWN', None)
  # The following definition for values can be found here:
  # https://cs.chromium.org/chromium/src/ui/base/page_transition_types.h
  _PAGE_TRANSITIONS = {
      0: ('LINK', 'User clicked a link'),
      1: ('TYPED', 'User typed the URL in the URL bar'),
      2: ('AUTO_BOOKMARK', 'Got through a suggestion in the UI'),
      3: ('AUTO_SUBFRAME',
          ('Content automatically loaded in a non-toplevel frame - user may not'
           'realize')),
      4: ('MANUAL_SUBFRAME', 'Subframe explicitly requested by the user'),
      5: ('GENERATED',
          ('User typed in the URL bar and selected an entry from the list - '
           'such as a search bar')),
      6: ('START_PAGE', 'The start page of the browser'),
      7: ('FORM_SUBMIT', 'A form the user has submitted values to'),
      8: ('RELOAD',
          ('The user reloaded the page, eg by hitting the reload button or '
           'restored a session')),
      9: ('KEYWORD',
          ('URL what was generated from a replaceable keyword other than the '
           'default search provider')),
      10: ('KEYWORD_GENERATED',
           'Corresponds to a visit generated from a KEYWORD')}

  # The following is the values for the source enum found in the visit_source
  # table and describes where a record originated from (if it originates from a
  # different storage than locally generated). The source can be found here:
  # https://cs.chromium.org/chromium/src/ui/app_list/search/history_types.h
  _VISIT_SOURCE = {
      0: 'SOURCE_SYNCED',
      1: 'SOURCE_BROWSED',
      2: 'SOURCE_EXTENSION',
      3: 'SOURCE_FIREFOX_IMPORTED',
      4: 'SOURCE_IE_IMPORTED',
      5: 'SOURCE_SAFARI_IMPORTED'}

  # pylint: disable=unused-argument
  def GetMessages(self, formatter_mediator, event_data):
    """Determines the formatted message strings for the event data.

    Args:
      formatter_mediator (FormatterMediator): mediates the interactions between
          formatters and other components, such as storage and Windows EventLog
          resources.
      event_data (EventData): event data.

    Returns:
      tuple(str, str): formatted message string and short message string.

    Raises:
      WrongFormatter: if the event data cannot be formatted by the formatter.
    """
    if self.DATA_TYPE != event_data.data_type:
      raise errors.WrongFormatter('Unsupported data type: {0:s}.'.format(
          event_data.data_type))

    event_values = event_data.CopyToDict()

    page_transition_type = event_values.get('page_transition_type', None)
    if page_transition_type is not None:
      page_transition, page_transition_long = self._PAGE_TRANSITIONS.get(
          page_transition_type, self._UNKNOWN_PAGE_TRANSITION)

      if page_transition_long:
        event_values['page_transition'] = '{0:s} - {1:s}'.format(
            page_transition, page_transition_long)
      else:
        event_values['page_transition'] = page_transition

    visit_source = event_values.get('visit_source', None)
    if visit_source is not None:
      event_values['visit_source'] = self._VISIT_SOURCE.get(
          visit_source, 'UNKNOWN')

    extras = []

    url_hidden = event_values.get('url_hidden', False)
    if url_hidden:
      extras.append('(url hidden)')

    typed_count = event_values.get('typed_count', 0)
    if typed_count == 0:
      extras.append('(URL not typed directly - no typed count)')
    elif typed_count == 1:
      extras.append('(type count {0:d} time)'.format(typed_count))
    else:
      extras.append('(type count {0:d} times)'.format(typed_count))

    event_values['extra'] = ' '.join(extras)

    return self._ConditionalFormatMessages(event_values)


manager.FormattersManager.RegisterFormatters([
    ChromeFileDownloadFormatter, ChromePageVisitedFormatter])



# -*- coding: utf-8 -*-
"""The Windows installation event formatter."""

from __future__ import unicode_literals

from plaso.formatters import interface
from plaso.formatters import manager


class WindowsRegistryInstallationEventFormatter(
    interface.ConditionalEventFormatter):
  """Formatter for a Windows installation event."""

  DATA_TYPE = 'windows:registry:installation'

  FORMAT_STRING_PIECES = [
      '{product_name}',
      '{version}',
      '{build_number}',
      '{service_pack}',
      'Owner: {owner}',
      'Origin: {key_path}']

  FORMAT_STRING_SHORT_PIECES = [
      '{product_name}',
      '{version}',
      '{build_number}',
      '{service_pack}',
      'Origin: {key_path}']

  SOURCE_LONG = 'System'
  SOURCE_SHORT = 'LOG'


manager.FormattersManager.RegisterFormatter(
    WindowsRegistryInstallationEventFormatter)



# -*- coding: utf-8 -*-
"""The Windows Registry plugin interface."""

from __future__ import unicode_literals

import abc

from plaso.parsers import plugins


class BaseWindowsRegistryKeyFilter(object):
  """The Windows Registry key filter interface."""

  @property
  def key_paths(self):
    """list[str]: key paths defined by the filter."""
    return []

  # pylint: disable=redundant-returns-doc
  @abc.abstractmethod
  def Match(self, registry_key):
    """Determines if a Windows Registry key matches the filter.

    Args:
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.

    Returns:
      bool: True if the keys match.
    """


class WindowsRegistryKeyPathFilter(BaseWindowsRegistryKeyFilter):
  """Windows Registry key path filter."""

  _CONTROL_SET_PREFIX = (
      'HKEY_LOCAL_MACHINE\\System\\CurrentControlSet')

  # This list must be ordered with most specific matches first.
  _WOW64_PREFIXES = [
      'HKEY_CURRENT_USER\\Software\\Classes',
      'HKEY_CURRENT_USER\\Software',
      'HKEY_LOCAL_MACHINE\\Software\\Classes',
      'HKEY_LOCAL_MACHINE\\Software']

  def __init__(self, key_path):
    """Initializes a Windows Registry key filter.

    Args:
      key_path (str): key path.
    """
    super(WindowsRegistryKeyPathFilter, self).__init__()

    key_path.rstrip('\\')
    self._key_path = key_path

    key_path = key_path.upper()
    self._key_path_upper = key_path

    self._wow64_key_path = None
    self._wow64_key_path_upper = None

    if key_path.startswith(self._CONTROL_SET_PREFIX.upper()):
      self._key_path_prefix, _, self._key_path_suffix = key_path.partition(
          'CurrentControlSet'.upper())

    else:
      self._key_path_prefix = None
      self._key_path_suffix = None

      # Handle WoW64 Windows Registry key redirection.
      # Also see:
      # https://msdn.microsoft.com/en-us/library/windows/desktop/
      # ms724072%28v=vs.85%29.aspx
      # https://msdn.microsoft.com/en-us/library/windows/desktop/
      # aa384253(v=vs.85).aspx
      wow64_prefix = None
      for key_path_prefix in self._WOW64_PREFIXES:
        if key_path.startswith(key_path_prefix.upper()):
          wow64_prefix = key_path_prefix
          break

      if wow64_prefix:
        key_path_suffix = self._key_path[len(wow64_prefix):]
        self._wow64_key_path = '\\'.join([wow64_prefix, 'Wow6432Node'])

        if key_path_suffix.startswith('\\'):
          key_path_suffix = key_path_suffix[1:]
        if key_path_suffix:
          self._wow64_key_path = '\\'.join([
              self._wow64_key_path, key_path_suffix])
        self._wow64_key_path_upper = self._wow64_key_path.upper()

  @property
  def key_paths(self):
    """List of key paths defined by the filter."""
    if self._wow64_key_path:
      return [self._key_path, self._wow64_key_path]
    return [self._key_path]

  def Match(self, registry_key):
    """Determines if a Windows Registry key matches the filter.

    Args:
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.

    Returns:
      bool: True if the keys match.
    """
    key_path = registry_key.path.upper()
    if self._key_path_prefix and self._key_path_suffix:
      if (key_path.startswith(self._key_path_prefix) and
          key_path.endswith(self._key_path_suffix)):

        key_path_segment = key_path[
            len(self._key_path_prefix):-len(self._key_path_suffix)]
        if key_path_segment.startswith('ControlSet'.upper()):
          try:
            control_set = int(key_path_segment[10:], 10)
          except ValueError:
            control_set = None

          # TODO: check if control_set is in bounds.
          return control_set is not None

    return key_path in (self._key_path_upper, self._wow64_key_path_upper)


class WindowsRegistryKeyPathPrefixFilter(BaseWindowsRegistryKeyFilter):
  """Windows Registry key path prefix filter."""

  def __init__(self, key_path_prefix):
    """Initializes a Windows Registry key filter.

    Args:
      key_path_prefix (str): the key path prefix.
    """
    super(WindowsRegistryKeyPathPrefixFilter, self).__init__()
    self._key_path_prefix = key_path_prefix

  def Match(self, registry_key):
    """Determines if a Windows Registry key matches the filter.

    Args:
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.

    Returns:
      bool: True if the keys match.
    """
    return registry_key.path.startswith(self._key_path_prefix)


class WindowsRegistryKeyPathSuffixFilter(BaseWindowsRegistryKeyFilter):
  """Windows Registry key path suffix filter."""

  def __init__(self, key_path_suffix):
    """Initializes a Windows Registry key filter.

    Args:
      key_path_suffix (str): the key path suffix.
    """
    super(WindowsRegistryKeyPathSuffixFilter, self).__init__()
    self._key_path_suffix = key_path_suffix

  def Match(self, registry_key):
    """Determines if a Windows Registry key matches the filter.

    Args:
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.

    Returns:
      bool: True if the keys match.
    """
    return registry_key.path.endswith(self._key_path_suffix)


class WindowsRegistryKeyWithValuesFilter(BaseWindowsRegistryKeyFilter):
  """Windows Registry key with values filter."""

  _EMPTY_SET = frozenset()

  def __init__(self, value_names):
    """Initializes a Windows Registry key filter.

    Args:
      value_names (list[str]): name of values that should be present in the key.
    """
    super(WindowsRegistryKeyWithValuesFilter, self).__init__()
    self._value_names = frozenset(value_names)

  def Match(self, registry_key):
    """Determines if a Windows Registry key matches the filter.

    Args:
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.

    Returns:
      bool: True if the keys match.
    """
    value_names = frozenset([
        registry_value.name for registry_value in registry_key.GetValues()])

    return self._value_names.issubset(value_names)


class WindowsRegistryPlugin(plugins.BasePlugin):
  """The Windows Registry plugin interface."""

  NAME = 'winreg_plugin'
  DESCRIPTION = 'Parser for Windows Registry value data.'

  # List of Windows Registry key filters (instances of
  # BaseWindowsRegistryKeyFilter) that should match for the plugin to
  # parse the Windows Registry key or its values.
  FILTERS = frozenset()

  def _GetValuesFromKey(self, registry_key, names_to_skip=None):
    """Retrieves the values from a Windows Registry key.

    Where:
    * the default value is represented as "(default)";
    * binary data values are represented as "(# bytes)", where # contains
          the number of bytes of the data;
    * empty values are represented as "(empty)".
    * empty multi value string values are represented as "[]".

    Args:
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.
      names_to_skip (Optional[list[str]]): names of values that should
          be skipped.

    Returns:
      dict[str, object]: names and data of the values in the key.
    """
    names_to_skip = [name.lower() for name in names_to_skip or []]

    values_dict = {}
    for registry_value in registry_key.GetValues():
      value_name = registry_value.name or '(default)'
      if value_name.lower() in names_to_skip:
        continue

      if registry_value.data is None:
        value_string = '(empty)'
      else:
        value_object = registry_value.GetDataAsObject()

        if registry_value.DataIsMultiString():
          value_string = '[{0:s}]'.format(', '.join([
              value for value in value_object or []]))

        elif (registry_value.DataIsInteger() or
              registry_value.DataIsString()):
          value_string = '{0!s}'.format(value_object)

        else:
          # Represent remaining types like REG_BINARY and
          # REG_RESOURCE_REQUIREMENT_LIST.
          value_string = '({0:d} bytes)'.format(len(value_object))

      data_type_string = registry_value.data_type_string

      # Correct typo in dfWinReg < 20190620
      if data_type_string == 'REG_RESOURCE_REQUIREMENT_LIST':
        data_type_string = 'REG_RESOURCE_REQUIREMENTS_LIST'

      value_string = '[{0:s}] {1:s}'.format(data_type_string, value_string)
      values_dict[value_name] = value_string

    return values_dict

  # pylint: disable=arguments-differ
  @abc.abstractmethod
  def ExtractEvents(self, parser_mediator, registry_key, **kwargs):
    """Extracts events from a Windows Registry key.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.
    """

  # TODO: merge with UpdateChainAndProcess, also requires changes to tests.
  def Process(self, parser_mediator, registry_key, **kwargs):
    """Processes a Windows Registry key or value.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.

    Raises:
      ValueError: If the Windows Registry key is not set.
    """
    if registry_key is None:
      raise ValueError('Windows Registry key is not set.')

    # This will raise if unhandled keyword arguments are passed.
    super(WindowsRegistryPlugin, self).Process(parser_mediator, **kwargs)

    self.ExtractEvents(parser_mediator, registry_key, **kwargs)

  # pylint: disable=arguments-differ
  def UpdateChainAndProcess(self, parser_mediator, registry_key, **kwargs):
    """Updates the parser chain and processes a Windows Registry key or value.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.

    Raises:
      ValueError: If the Windows Registry key is not set.
    """
    parser_mediator.AppendToParserChain(self)
    try:
      self.Process(parser_mediator, registry_key, **kwargs)
    finally:
      parser_mediator.PopFromParserChain()



# -*- coding: utf-8 -*-
"""The hashers CLI arguments helper."""

from __future__ import unicode_literals

from plaso.cli import tools
from plaso.cli.helpers import interface
from plaso.cli.helpers import manager
from plaso.lib import errors


class HashersArgumentsHelper(interface.ArgumentsHelper):
  """Hashers CLI arguments helper."""

  NAME = 'hashers'
  DESCRIPTION = 'Hashers command line arguments.'

  _DEFAULT_HASHER_STRING = 'sha256'

  @classmethod
  def AddArguments(cls, argument_group):
    """Adds command line arguments to an argument group.

    This function takes an argument parser or an argument group object and adds
    to it all the command line arguments this helper supports.

    Args:
      argument_group (argparse._ArgumentGroup|argparse.ArgumentParser):
          argparse group.
    """
    argument_group.add_argument(
        '--hasher_file_size_limit', '--hasher-file-size-limit',
        dest='hasher_file_size_limit', type=int, action='store', default=0,
        metavar='SIZE', help=(
            'Define the maximum file size in bytes that hashers should '
            'process. Any larger file will be skipped. A size of 0 represents '
            'no limit.'))

    argument_group.add_argument(
        '--hashers', dest='hashers', type=str, action='store',
        default=cls._DEFAULT_HASHER_STRING, metavar='HASHER_LIST', help=(
            'Define a list of hashers to use by the tool. This is a comma '
            'separated list where each entry is the name of a hasher, such as '
            '"md5,sha256". "all" indicates that all hashers should be '
            'enabled. "none" disables all hashers. Use "--hashers list" or '
            '"--info" to list the available hashers.'))

  @classmethod
  def ParseOptions(cls, options, configuration_object):
    """Parses and validates options.

    Args:
      options (argparse.Namespace): parser options.
      configuration_object (CLITool): object to be configured by the argument
          helper.

    Raises:
      BadConfigObject: when the configuration object is of the wrong type.
      BadConfigOption: when a configuration parameter fails validation.
    """
    if not isinstance(configuration_object, tools.CLITool):
      raise errors.BadConfigObject(
          'Configuration object is not an instance of CLITool')

    hashers = cls._ParseStringOption(
        options, 'hashers', default_value=cls._DEFAULT_HASHER_STRING)

    hasher_file_size_limit = cls._ParseNumericOption(
        options, 'hasher_file_size_limit', default_value=0)

    # TODO: validate hasher names.

    if hasher_file_size_limit < 0:
      raise errors.BadConfigOption(
          'Invalid hasher file size limit value cannot be negative.')

    setattr(configuration_object, '_hasher_names_string', hashers)
    setattr(
        configuration_object, '_hasher_file_size_limit', hasher_file_size_limit)


manager.ArgumentHelperManager.RegisterHelper(HashersArgumentsHelper)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the YAML-based filter file."""

from __future__ import unicode_literals

import io
import unittest

from plaso.engine import yaml_filter_file
from plaso.lib import errors

from tests import test_lib as shared_test_lib


class YAMLFilterFileTest(shared_test_lib.BaseTestCase):
  """Tests for the YAML-based filter file."""

  # pylint: disable=protected-access

  def testReadFilterDefinition(self):
    """Tests the _ReadFilterDefinition function."""
    test_filter_file = yaml_filter_file.YAMLFilterFile()

    path_filter = test_filter_file._ReadFilterDefinition({
        'type': 'include', 'paths': ['/usr/bin']})

    self.assertIsNotNone(path_filter)
    self.assertIsNone(path_filter.description)
    self.assertEqual(path_filter.filter_type, 'include')
    self.assertEqual(path_filter.path_separator, '/')
    self.assertEqual(path_filter.paths, ['/usr/bin'])

    with self.assertRaises(errors.ParseError):
      test_filter_file._ReadFilterDefinition({})

    with self.assertRaises(errors.ParseError):
      test_filter_file._ReadFilterDefinition({'type': 'bogus'})

    with self.assertRaises(errors.ParseError):
      test_filter_file._ReadFilterDefinition({'type': 'include'})

    with self.assertRaises(errors.ParseError):
      test_filter_file._ReadFilterDefinition({'bogus': 'error'})

  def testReadFromFileObject(self):
    """Tests the _ReadFromFileObject function."""
    test_file_path = self._GetTestFilePath(['filter_files', 'format_test.yaml'])
    self._SkipIfPathNotExists(test_file_path)

    test_filter_file = yaml_filter_file.YAMLFilterFile()
    with io.open(test_file_path, 'r', encoding='utf-8') as file_object:
      path_filters = list(test_filter_file._ReadFromFileObject(file_object))

    self.assertEqual(len(path_filters), 3)

  def testReadFromFile(self):
    """Tests the ReadFromFile function."""
    test_file_path = self._GetTestFilePath(['filter_files', 'format_test.yaml'])
    self._SkipIfPathNotExists(test_file_path)

    test_filter_file = yaml_filter_file.YAMLFilterFile()
    path_filters = test_filter_file.ReadFromFile(test_file_path)

    self.assertEqual(len(path_filters), 3)

    self.assertEqual(path_filters[0].path_separator, '/')
    self.assertEqual(path_filters[0].paths, ['/usr/bin'])

    self.assertEqual(path_filters[1].path_separator, '\\')
    self.assertEqual(path_filters[1].paths, ['\\\\Windows\\\\System32'])


if __name__ == '__main__':
  unittest.main()
